@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -21,6 +21,9 @@
 #ifdef HAVE_PGSQL
 #include <dhcpsrv/pgsql_lease_mgr.h>
 #endif
+#ifdef HAVE_CQL
+#include <dhcpsrv/cql_lease_mgr.h>
+#endif
 #include <dhcpsrv/memfile_lease_mgr.h>
 
 #include <sstream>
@@ -484,6 +487,9 @@ DControllerBase::getVersion(bool extended) {
 #endif
 #ifdef HAVE_PGSQL
         tmp << isc::dhcp::PgSqlLeaseMgr::getDBVersion() << std::endl;
+#endif
+#ifdef HAVE_CQL
+        tmp << isc::dhcp::CqlLeaseMgr::getDBVersion() << std::endl;
 #endif
         tmp << isc::dhcp::Memfile_LeaseMgr::getDBVersion();
 
