@@ -1,17 +1,16 @@
-// Copyright (C) 2013-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
-
+#include <asiolink/asio_wrapper.h>
+#include <asiolink/udp_endpoint.h>
 #include <d2/d2_cfg_mgr.h>
 #include <dns/opcode.h>
 #include <dns/messagerenderer.h>
 #include <nc_test_utils.h>
-#include <boost/asio.hpp>
-#include <asiolink/udp_endpoint.h>
 #include <util/encode/base64.h>
 
 #include <gtest/gtest.h>
