@@ -1,4 +1,4 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -17,6 +17,9 @@
 #include <dhcp/tests/iface_mgr_test_config.h>
 #include <dhcp/option.h>
 #include <asiolink/io_address.h>
+#include <dhcp4/tests/marker_file.h>
+#include <dhcp4/tests/test_libraries.h>
+
 #include <vector>
 
 using namespace std;
@@ -194,9 +197,13 @@ public:
     buffer4_receive_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("buffer4_receive");
 
-        callout_handle.getArgument("query4", callback_pkt4_);
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ = callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
         return (0);
     }
 
@@ -239,9 +246,14 @@ public:
     pkt4_receive_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("pkt4_receive");
 
-        callout_handle.getArgument("query4", callback_pkt4_);
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ = callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -304,11 +316,20 @@ public:
     pkt4_send_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("pkt4_send");
 
-        callout_handle.getArgument("response4", callback_pkt4_);
+        callout_handle.getArgument("response4", callback_resp_pkt4_);
 
-        callout_handle.getArgument("query4", callback_pkt4_query_);
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ = callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
+
+        if (callback_resp_pkt4_) {
+            callback_resp_options_copy_ = callback_resp_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -369,9 +390,14 @@ public:
     buffer4_send_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("buffer4_send");
 
-        callout_handle.getArgument("response4", callback_pkt4_);
+        callout_handle.getArgument("response4", callback_resp_pkt4_);
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_resp_pkt4_) {
+            callback_resp_options_copy_ = callback_resp_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -409,11 +435,16 @@ public:
     subnet4_select_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("subnet4_select");
 
-        callout_handle.getArgument("query4", callback_pkt4_);
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
         callout_handle.getArgument("subnet4", callback_subnet4_);
         callout_handle.getArgument("subnet4collection", callback_subnet4collection_);
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ = callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -447,10 +478,15 @@ public:
     lease4_release_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("lease4_release");
 
-        callout_handle.getArgument("query4", callback_pkt4_);
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
         callout_handle.getArgument("lease4", callback_lease4_);
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ = callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -461,12 +497,18 @@ public:
     lease4_renew_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("lease4_renew");
 
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
         callout_handle.getArgument("subnet4", callback_subnet4_);
         callout_handle.getArgument("lease4", callback_lease4_);
         callout_handle.getArgument("hwaddr", callback_hwaddr_);
         callout_handle.getArgument("clientid", callback_clientid_);
 
         callback_argument_names_ = callout_handle.getArgumentNames();
+
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ = callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -477,9 +519,13 @@ public:
     static int
     lease4_decline_callout(CalloutHandle& callout_handle) {
         callback_name_ = string("lease4_decline");
-        callout_handle.getArgument("query4", callback_pkt4_);
+        callout_handle.getArgument("query4", callback_qry_pkt4_);
         callout_handle.getArgument("lease4", callback_lease4_);
 
+        if (callback_qry_pkt4_) {
+            callback_qry_options_copy_ = callback_qry_pkt4_->isCopyRetrievedOptions();
+        }
+
         return (0);
     }
 
@@ -497,14 +543,16 @@ public:
     /// resets buffers used to store data received by callouts
     void resetCalloutBuffers() {
         callback_name_ = string("");
-        callback_pkt4_.reset();
-        callback_pkt4_query_.reset();
+        callback_qry_pkt4_.reset();
+        callback_qry_pkt4_.reset();
         callback_lease4_.reset();
         callback_hwaddr_.reset();
         callback_clientid_.reset();
         callback_subnet4_.reset();
         callback_subnet4collection_ = NULL;
         callback_argument_names_.clear();
+        callback_qry_options_copy_ = false;
+        callback_resp_options_copy_ = false;
     }
 
     /// pointer to Dhcpv4Srv that is used in tests
@@ -515,14 +563,11 @@ public:
     /// String name of the received callout
     static string callback_name_;
 
-    /// Pkt4 structure returned in the callout
-    static Pkt4Ptr callback_pkt4_;
+    /// Client/query Pkt4 structure returned in the callout
+    static Pkt4Ptr callback_qry_pkt4_;
 
-    /// @brief Pkt4 structure returned in the callout
-    ///
-    /// pkt4_send hook now passes both the query and the response,
-    /// so we actually need two fields.
-    static Pkt4Ptr callback_pkt4_query_;
+    /// Server/response Pkt4 structure returned in the callout
+    static Pkt4Ptr callback_resp_pkt4_;
 
     /// Lease4 structure returned in the callout
     static Lease4Ptr callback_lease4_;
@@ -541,19 +586,61 @@ public:
 
     /// A list of all received arguments
     static vector<string> callback_argument_names_;
+
+    /// Flag indicating if copying retrieved options was enabled for
+    /// a query during callout execution.
+    static bool callback_qry_options_copy_;
+
+    /// Flag indicating if copying retrieved options was enabled for
+    /// a response during callout execution.
+    static bool callback_resp_options_copy_;
+
 };
 
 // The following fields are used in testing pkt4_receive_callout.
 // See fields description in the class for details
 string HooksDhcpv4SrvTest::callback_name_;
-Pkt4Ptr HooksDhcpv4SrvTest::callback_pkt4_;
-Pkt4Ptr HooksDhcpv4SrvTest::callback_pkt4_query_;
+Pkt4Ptr HooksDhcpv4SrvTest::callback_qry_pkt4_;
+Pkt4Ptr HooksDhcpv4SrvTest::callback_resp_pkt4_;
 Subnet4Ptr HooksDhcpv4SrvTest::callback_subnet4_;
 HWAddrPtr HooksDhcpv4SrvTest::callback_hwaddr_;
 ClientIdPtr HooksDhcpv4SrvTest::callback_clientid_;
 Lease4Ptr HooksDhcpv4SrvTest::callback_lease4_;
 const Subnet4Collection* HooksDhcpv4SrvTest::callback_subnet4collection_;
 vector<string> HooksDhcpv4SrvTest::callback_argument_names_;
+bool HooksDhcpv4SrvTest::callback_qry_options_copy_;
+bool HooksDhcpv4SrvTest::callback_resp_options_copy_;
+
+/// @brief Fixture class used to do basic library load/unload tests
+class LoadUnloadDhcpv4SrvTest : public ::testing::Test {
+public:
+    /// @brief Pointer to the tested server object
+    boost::shared_ptr<NakedDhcpv4Srv> server_;
+
+    LoadUnloadDhcpv4SrvTest() {
+        reset();
+    }
+
+    /// @brief Destructor
+    ~LoadUnloadDhcpv4SrvTest() {
+        server_.reset();
+        reset();
+    };
+
+    /// @brief Reset hooks data
+    ///
+    /// Resets the data for the hooks-related portion of the test by ensuring
+    /// that no libraries are loaded and that any marker files are deleted.
+    void reset() {
+        // Unload any previously-loaded libraries.
+        HooksManager::unloadLibraries();
+
+        // Get rid of any marker files.
+        static_cast<void>(remove(LOAD_MARKER_FILE));
+        static_cast<void>(remove(UNLOAD_MARKER_FILE));
+        CfgMgr::instance().clear();
+    }
+};
 
 // Checks if callouts installed on pkt4_receive are indeed called and the
 // all necessary parameters are passed.
@@ -584,13 +671,16 @@ TEST_F(HooksDhcpv4SrvTest, Buffer4ReceiveSimple) {
     EXPECT_EQ("buffer4_receive", callback_name_);
 
     // Check that pkt4 argument passing was successful and returned proper value
-    EXPECT_TRUE(callback_pkt4_.get() == dis.get());
+    EXPECT_TRUE(callback_qry_pkt4_.get() == dis.get());
 
     // Check that all expected parameters are there
     vector<string> expected_argument_names;
     expected_argument_names.push_back(string("query4"));
 
     EXPECT_TRUE(expected_argument_names == callback_argument_names_);
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // Checks if callouts installed on buffer4_receive is able to change
@@ -689,13 +779,16 @@ TEST_F(HooksDhcpv4SrvTest, pkt4ReceiveSimple) {
     EXPECT_EQ("pkt4_receive", callback_name_);
 
     // check that pkt4 argument passing was successful and returned proper value
-    EXPECT_TRUE(callback_pkt4_.get() == sol.get());
+    EXPECT_TRUE(callback_qry_pkt4_.get() == sol.get());
 
     // Check that all expected parameters are there
     vector<string> expected_argument_names;
     expected_argument_names.push_back(string("query4"));
 
     EXPECT_TRUE(expected_argument_names == callback_argument_names_);
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // Checks if callouts installed on pkt4_received is able to change
@@ -819,12 +912,12 @@ TEST_F(HooksDhcpv4SrvTest, pkt4SendSimple) {
     Pkt4Ptr adv = srv_->fake_sent_.front();
 
     // Check that pkt4 argument passing was successful and returned proper value
-    ASSERT_TRUE(callback_pkt4_);
-    EXPECT_TRUE(callback_pkt4_.get() == adv.get());
+    ASSERT_TRUE(callback_resp_pkt4_);
+    EXPECT_TRUE(callback_resp_pkt4_.get() == adv.get());
 
     // That that the query4 argument was correctly set to the Discover we sent.
-    ASSERT_TRUE(callback_pkt4_query_);
-    EXPECT_TRUE(callback_pkt4_query_.get() == sol.get());
+    ASSERT_TRUE(callback_qry_pkt4_);
+    EXPECT_TRUE(callback_qry_pkt4_.get() == sol.get());
 
     // Check that all expected parameters are there
     vector<string> expected_argument_names;
@@ -833,6 +926,10 @@ TEST_F(HooksDhcpv4SrvTest, pkt4SendSimple) {
     sort(callback_argument_names_.begin(), callback_argument_names_.end());
     sort(expected_argument_names.begin(), expected_argument_names.end());
     EXPECT_TRUE(expected_argument_names == callback_argument_names_);
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_qry_options_copy_);
+    EXPECT_TRUE(callback_resp_options_copy_);
 }
 
 // Checks if callouts installed on pkt4_send is able to change
@@ -968,12 +1065,15 @@ TEST_F(HooksDhcpv4SrvTest, buffer4SendSimple) {
     Pkt4Ptr adv = srv_->fake_sent_.front();
 
     // Check that pkt4 argument passing was successful and returned proper value
-    EXPECT_TRUE(callback_pkt4_.get() == adv.get());
+    EXPECT_TRUE(callback_resp_pkt4_.get() == adv.get());
 
     // Check that all expected parameters are there
     vector<string> expected_argument_names;
     expected_argument_names.push_back(string("response4"));
     EXPECT_TRUE(expected_argument_names == callback_argument_names_);
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_resp_options_copy_);
 }
 
 // Checks if callouts installed on buffer4_send are indeed called and that
@@ -1090,7 +1190,7 @@ TEST_F(HooksDhcpv4SrvTest, subnet4SelectSimple) {
     EXPECT_EQ("subnet4_select", callback_name_);
 
     // Check that pkt4 argument passing was successful and returned proper value
-    EXPECT_TRUE(callback_pkt4_.get() == sol.get());
+    EXPECT_TRUE(callback_qry_pkt4_.get() == sol.get());
 
     const Subnet4Collection* exp_subnets =
         CfgMgr::instance().getCurrentCfg()->getCfgSubnets4()->getAll();
@@ -1107,6 +1207,9 @@ TEST_F(HooksDhcpv4SrvTest, subnet4SelectSimple) {
     // Compare that the available subnets are reported as expected
     EXPECT_TRUE((*exp_subnets)[0].get() == (*callback_subnet4collection_)[0].get());
     EXPECT_TRUE((*exp_subnets)[1].get() == (*callback_subnet4collection_)[1].get());
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // This test checks if callout installed on subnet4_select hook point can pick
@@ -1240,6 +1343,10 @@ TEST_F(HooksDhcpv4SrvTest, lease4RenewSimple) {
     // Check that the callback called is indeed the one we installed
     EXPECT_EQ("lease4_renew", callback_name_);
 
+    // Check that query4 argument passing was successful and
+    // returned proper value
+    EXPECT_TRUE(callback_qry_pkt4_.get() == req.get());
+
     // Check that hwaddr parameter is passed properly
     ASSERT_TRUE(callback_hwaddr_);
     EXPECT_TRUE(*callback_hwaddr_ == *req->getHWAddr());
@@ -1254,6 +1361,7 @@ TEST_F(HooksDhcpv4SrvTest, lease4RenewSimple) {
 
     // Check if all expected parameters were really received
     vector<string> expected_argument_names;
+    expected_argument_names.push_back("query4");
     expected_argument_names.push_back("subnet4");
     expected_argument_names.push_back("clientid");
     expected_argument_names.push_back("hwaddr");
@@ -1263,6 +1371,9 @@ TEST_F(HooksDhcpv4SrvTest, lease4RenewSimple) {
     EXPECT_TRUE(callback_argument_names_ == expected_argument_names);
 
     EXPECT_TRUE(LeaseMgrFactory::instance().deleteLease(addr));
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // This test verifies that a callout installed on lease4_renew can trigger
@@ -1410,7 +1521,7 @@ TEST_F(HooksDhcpv4SrvTest, lease4ReleaseSimple) {
     EXPECT_EQ("lease4_release", callback_name_);
 
     // Check that pkt4 argument passing was successful and returned proper value
-    EXPECT_TRUE(callback_pkt4_.get() == rel.get());
+    EXPECT_TRUE(callback_qry_pkt4_.get() == rel.get());
 
     // Check if all expected parameters were really received
     vector<string> expected_argument_names;
@@ -1419,6 +1530,9 @@ TEST_F(HooksDhcpv4SrvTest, lease4ReleaseSimple) {
     sort(callback_argument_names_.begin(), callback_argument_names_.end());
     sort(expected_argument_names.begin(), expected_argument_names.end());
     EXPECT_TRUE(callback_argument_names_ == expected_argument_names);
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // This test verifies that skip flag returned by a callout installed on the
@@ -1506,15 +1620,15 @@ TEST_F(HooksDhcpv4SrvTest, HooksDecline) {
     // Verifying DHCPDECLINE is a bit tricky, as it is created somewhere in
     // acquireAndDecline. We'll just verify that it's really a DECLINE
     // and that its address is equal to what we have in LeaseMgr.
-    ASSERT_TRUE(callback_pkt4_);
+    ASSERT_TRUE(callback_qry_pkt4_);
     ASSERT_TRUE(callback_lease4_);
 
     // Check that it's the proper packet that was reported.
-    EXPECT_EQ(DHCPDECLINE, callback_pkt4_->getType());
+    EXPECT_EQ(DHCPDECLINE, callback_qry_pkt4_->getType());
 
     // Extract the address being declined.
     OptionCustomPtr opt_declined_addr = boost::dynamic_pointer_cast<
-        OptionCustom>(callback_pkt4_->getOption(DHO_DHCP_REQUESTED_ADDRESS));
+        OptionCustom>(callback_qry_pkt4_->getOption(DHO_DHCP_REQUESTED_ADDRESS));
     ASSERT_TRUE(opt_declined_addr);
     IOAddress addr(opt_declined_addr->readAddress());
 
@@ -1527,6 +1641,9 @@ TEST_F(HooksDhcpv4SrvTest, HooksDecline) {
     // the lease manager) all match.
     EXPECT_EQ(addr, from_mgr->addr_);
     EXPECT_EQ(addr, callback_lease4_->addr_);
+
+    // Pkt passed to a callout must be configured to copy retrieved options.
+    EXPECT_TRUE(callback_qry_options_copy_);
 }
 
 // Checks that decline4 hook is able to drop the packet.
@@ -1550,15 +1667,15 @@ TEST_F(HooksDhcpv4SrvTest, HooksDeclineDrop) {
     // Verifying DHCPDECLINE is a bit tricky, as it is created somewhere in
     // acquireAndDecline. We'll just verify that it's really a DECLINE
     // and that its address is equal to what we have in LeaseMgr.
-    ASSERT_TRUE(callback_pkt4_);
+    ASSERT_TRUE(callback_qry_pkt4_);
     ASSERT_TRUE(callback_lease4_);
 
     // Check that it's the proper packet that was reported.
-    EXPECT_EQ(DHCPDECLINE, callback_pkt4_->getType());
+    EXPECT_EQ(DHCPDECLINE, callback_qry_pkt4_->getType());
 
     // Extract the address being declined.
     OptionCustomPtr opt_declined_addr = boost::dynamic_pointer_cast<
-        OptionCustom>(callback_pkt4_->getOption(DHO_DHCP_REQUESTED_ADDRESS));
+        OptionCustom>(callback_qry_pkt4_->getOption(DHO_DHCP_REQUESTED_ADDRESS));
     ASSERT_TRUE(opt_declined_addr);
     IOAddress addr(opt_declined_addr->readAddress());
 
@@ -1573,3 +1690,42 @@ TEST_F(HooksDhcpv4SrvTest, HooksDeclineDrop) {
     EXPECT_EQ(addr, from_mgr->addr_);
     EXPECT_EQ(addr, callback_lease4_->addr_);
 }
+
+
+// Verifies that libraries are unloaded by server destruction
+// The callout libraries write their library index number to a marker
+// file upon load and unload, making it simple to test whether or not
+// the load and unload callouts have been invoked.
+TEST_F(LoadUnloadDhcpv4SrvTest, unloadLibaries) {
+
+    ASSERT_NO_THROW(server_.reset(new NakedDhcpv4Srv()));
+
+    // Ensure no marker files to start with.
+    ASSERT_FALSE(checkMarkerFileExists(LOAD_MARKER_FILE));
+    ASSERT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
+
+    // Load the test libraries
+    HookLibsCollection libraries;
+    libraries.push_back(make_pair(std::string(CALLOUT_LIBRARY_1),
+                                  ConstElementPtr()));
+    libraries.push_back(make_pair(std::string(CALLOUT_LIBRARY_2),
+
+                                  ConstElementPtr()));
+    ASSERT_TRUE(HooksManager::loadLibraries(libraries));
+
+    // Verify that they load functions created the LOAD_MARKER_FILE
+    // and that it's contents are correct: "12" - the first library
+    // appends "1" to the file, the second appends "2"). Also
+    // check that the unload marker file does not yet exist.
+    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "12"));
+    EXPECT_FALSE(checkMarkerFileExists(UNLOAD_MARKER_FILE));
+
+    // Destroy the server, instance which should unload the libraries.
+    server_.reset();
+
+    // Check that the libraries were unloaded. The libraries are
+    // unloaded in the reverse order to which they are loaded, and
+    // this should be reflected in the unload file.
+    EXPECT_TRUE(checkMarkerFile(UNLOAD_MARKER_FILE, "21"));
+    EXPECT_TRUE(checkMarkerFile(LOAD_MARKER_FILE, "12"));
+}
