@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -20,6 +20,7 @@
 #include <stdlib.h>
 #include <stdint.h>
 #include <unistd.h>
+#include <fstream>
 
 
 using namespace std;
@@ -117,6 +118,8 @@ CommandOptions::reset() {
     mac_template_.assign(mac, mac + 6);
     duid_template_.clear();
     base_.clear();
+    mac_list_file_.clear();
+    mac_list_.clear();
     num_request_.clear();
     period_ = 0;
     drop_time_set_ = 0;
@@ -142,6 +145,7 @@ CommandOptions::reset() {
     diags_.clear();
     wrapped_.clear();
     server_name_.clear();
+    v6_relay_encapsulation_level_ = 0;
     generateDuidTemplate();
 }
 
@@ -205,10 +209,11 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
 
     std::ostringstream stream;
     stream << "perfdhcp";
+    int num_mac_list_files = 0;
 
     // In this section we collect argument values from command line
     // they will be tuned and validated elsewhere
-    while((opt = getopt(argc, argv, "hv46r:t:R:b:n:p:d:D:l:P:a:L:"
+    while((opt = getopt(argc, argv, "hv46A:r:t:R:b:n:p:d:D:l:P:a:L:M:"
                         "s:iBc1T:X:O:E:S:I:x:w:e:f:F:")) != -1) {
         stream << " -" << static_cast<char>(opt);
         if (optarg) {
@@ -219,6 +224,19 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
             use_first_ = true;
             break;
 
+        // Simulate DHCPv6 relayed traffic.
+        case 'A':
+            // @todo: At the moment we only support simulating a single relay
+            // agent. In the future we should extend it to up to 32.
+            // See comment in https://github.com/isc-projects/kea/pull/22#issuecomment-243405600
+            v6_relay_encapsulation_level_ =
+                static_cast<uint8_t>(positiveInteger("-A<encapusulation-level> must"
+                                                     " be a positive integer"));
+            if (v6_relay_encapsulation_level_ != 1) {
+                isc_throw(isc::InvalidParameter, "-A only supports 1 at the moment.");
+            }
+            break;
+
         case '4':
             check(ipversion_ == 6, "IP version already set to 6");
             ipversion_ = 4;
@@ -340,6 +358,13 @@ CommandOptions::initialize(int argc, char** argv, bool print_cmd_line) {
                   boost::lexical_cast<std::string>(std::numeric_limits<uint16_t>::max()));
             break;
 
+        case 'M':
+            check(num_mac_list_files >= 1, "only -M option can be specified");
+            num_mac_list_files++;
+            mac_list_file_ = std::string(optarg);
+            loadMacs();
+            break;
+
         case 'n':
             num_req = positiveInteger("value of num-request:"
                                       " -n<value> must be a positive integer");
@@ -554,7 +579,7 @@ CommandOptions::decodeBase(const std::string& base) {
 
     // Currently we only support mac and duid
     if ((b.substr(0, 4) == "mac=") || (b.substr(0, 6) == "ether=")) {
-        decodeMac(b);
+        decodeMacBase(b);
     } else if (b.substr(0, 5) == "duid=") {
         decodeDuid(b);
     } else {
@@ -565,7 +590,7 @@ CommandOptions::decodeBase(const std::string& base) {
 }
 
 void
-CommandOptions::decodeMac(const std::string& base) {
+CommandOptions::decodeMacBase(const std::string& base) {
     // Strip string from mac=
     size_t found = base.find('=');
     static const char* errmsg = "expected -b<base> format for"
@@ -685,14 +710,45 @@ CommandOptions::convertHexString(const std::string& text) const {
     return ui;
 }
 
+void CommandOptions::loadMacs() {
+  std::string line;
+  std::ifstream infile(mac_list_file_.c_str());
+  while (std::getline(infile, line)) {
+    check(decodeMacString(line), "invalid mac in input");
+  }
+}
+
+bool CommandOptions::decodeMacString(const std::string& line) {
+  // decode mac string into a vector of uint8_t returns true in case of error.
+  std::istringstream s(line);
+  std::string token;
+  std::vector<uint8_t> mac;
+  while(std::getline(s, token, ':')) {
+    // Convert token to byte value using std::istringstream
+    if (token.length() > 0) {
+      unsigned int ui = 0;
+      try {
+        // Do actual conversion
+        ui = convertHexString(token);
+      } catch (isc::InvalidParameter&) {
+        return (true);
+      }
+      // If conversion succeeded store byte value
+      mac.push_back(ui);
+    }
+  }
+  mac_list_.push_back(mac);
+  return (false);
+}
+
 void
 CommandOptions::validate() const {
     check((getIpVersion() != 4) && (isBroadcast() != 0),
           "-B is not compatible with IPv6 (-6)");
     check((getIpVersion() != 6) && (isRapidCommit() != 0),
           "-6 (IPv6) must be set to use -c");
-    check((getIpVersion() != 6) && (getRenewRate() !=0),
-          "-f<renew-rate> may be used with -6 (IPv6) only");
+    check(getIpVersion() == 4 && isUseRelayedV6(),
+          "Can't use -4 with -A, it's a V6 only option.");
     check((getIpVersion() != 6) && (getReleaseRate() != 0),
           "-F<release-rate> may be used with -6 (IPv6) only");
     check((getExchangeMode() == DO_SA) && (getNumRequests().size() > 1),
@@ -759,7 +815,8 @@ CommandOptions::validate() const {
     check((getTemplateFiles().size() < 2) && (getRequestedIpOffset() >= 0),
           "second/request -T<template-file> must be set to "
           "use -I<ip-offset>");
-
+    check((!getMacListFile().empty() && base_.size() > 0),
+          "Can't use -b with -M option");
 }
 
 void
@@ -873,6 +930,9 @@ CommandOptions::printCommandLine() const {
     if (use_first_) {
         std::cout << "use-first" << std::endl;
     }
+    if (!mac_list_file_.empty()) {
+        std::cout << "mac-list-file=" << mac_list_file_ << std::endl;
+    }
     for (size_t i = 0; i < template_file_.size(); ++i) {
         std::cout << "template-file[" << i << "]=" << template_file_[i] << std::endl;
     }
@@ -912,15 +972,16 @@ CommandOptions::printCommandLine() const {
 void
 CommandOptions::usage() const {
     std::cout <<
-        "perfdhcp [-hv] [-4|-6] [-e<lease-type>] [-r<rate>] [-f<renew-rate>]\n"
+        "perfdhcp [-hv] [-4|-6] [-A<encapusulation-level>] [-e<lease-type>]"
+        "         [-r<rate>] [-f<renew-rate>]\n"
         "         [-F<release-rate>] [-t<report>] [-R<range>] [-b<base>]\n"
         "         [-n<num-request>] [-p<test-period>] [-d<drop-time>]\n"
         "         [-D<max-drop>] [-l<local-addr|interface>] [-P<preload>]\n"
         "         [-a<aggressivity>] [-L<local-port>] [-s<seed>] [-i] [-B]\n"
-        "         [-c] [-1] [-T<template-file>] [-X<xid-offset>]\n"
-        "         [-O<random-offset] [-E<time-offset>] [-S<srvid-offset>]\n"
-        "         [-I<ip-offset>] [-x<diagnostic-selector>] [-w<wrapped>]\n"
-        "         [server]\n"
+        "         [-c] [-1] [-M<mac-list-file>] [-T<template-file>]\n"
+        "         [-X<xid-offset>] [-O<random-offset] [-E<time-offset>]\n"
+        "         [-S<srvid-offset>] [-I<ip-offset>] [-x<diagnostic-selector>]\n"
+        "         [-w<wrapped>] [server]\n"
         "\n"
         "The [server] argument is the name/address of the DHCP server to\n"
         "contact.  For DHCPv4 operation, exchanges are initiated by\n"
@@ -963,6 +1024,11 @@ CommandOptions::usage() const {
         "-E<time-offset>: Offset of the (DHCPv4) secs field / (DHCPv6)\n"
         "    elapsed-time option in the (second/request) template.\n"
         "    The value 0 disables it.\n"
+        "-f<renew-rate>: Rate at which DHCPv4 or DHCPv6 renew requests are sent\n"
+        "    to a server. This value is only valid when used in conjunction\n"
+        "    with the exchange rate (given by -r<rate>).  Furthermore the sum of\n"
+        "    this value and the release-rate (given by -F<rate) must be equal\n"
+        "    to or less than the exchange rate.\n"
         "-h: Print this help.\n"
         "-i: Do only the initial part of an exchange: DO or SA, depending on\n"
         "    whether -6 is given.\n"
@@ -976,6 +1042,11 @@ CommandOptions::usage() const {
         "    via which exchanges are initiated.\n"
         "-L<local-port>: Specify the local port to use\n"
         "    (the value 0 means to use the default).\n"
+        "-M<mac-list-file>: A text file containing a list of MAC addresses,\n"
+        "   one per line. If provided, a MAC address will be choosen randomly\n"
+        "   from this list for every new exchange. In the DHCPv6 case, MAC\n"
+        "   addresses are used to generate DUID-LLs. This parameter must not be\n"
+        "   used in conjunction with the -b parameter.\n"
         "-O<random-offset>: Offset of the last octet to randomize in the template.\n"
         "-P<preload>: Initiate first <preload> exchanges back to back at startup.\n"
         "-r<rate>: Initiate <rate> DORA/SARR (or if -i is given, DO/SA)\n"
@@ -1010,16 +1081,17 @@ CommandOptions::usage() const {
         "\n"
         "DHCPv6 only options:\n"
         "-c: Add a rapid commit option (exchanges will be SA).\n"
-        "-f<renew-rate>: Rate at which IPv6 Renew requests are sent to\n"
-        "    a server. This value is only valid when used in conjunction with\n"
-        "    the exchange rate (given by -r<rate>).  Furthermore the sum of\n"
-        "    this value and the release-rate (given by -F<rate) must be equal\n"
-        "    to or less than the exchange rate.\n"
         "-F<release-rate>: Rate at which IPv6 Release requests are sent to\n"
         "    a server. This value is only valid when used in conjunction with\n"
         "    the exchange rate (given by -r<rate>).  Furthermore the sum of\n"
         "    this value and the renew-rate (given by -f<rate) must be equal\n"
         "    to or less than the exchange rate.\n"
+        "-A<encapusulation-level>: Specifies that relayed traffic must be\n"
+        "    generated. The argument specifies the level of encapsulation, i.e.\n"
+        "    how many relay agents are simulated. Currently the only supported\n"
+        "    <encapsulation-level> value is 1, which means that the generated\n"
+        "    traffic is an equivalent of the traffic passing through a single\n"
+        "    relay agent.\n"
         "\n"
         "The remaining options are used only in conjunction with -r:\n"
         "\n"
