@@ -24,6 +24,9 @@ namespace perfdhcp {
 class CommandOptions : public boost::noncopyable {
 public:
 
+    /// @brief A vector holding MAC addresses.
+    typedef std::vector<std::vector<uint8_t> > MacAddrsVector;
+
     /// \brief A class encapsulating the type of lease being requested from the
     /// server.
     ///
@@ -269,11 +272,31 @@ public:
     /// \return true if server-iD to be taken from first package.
     bool isUseFirst() const { return use_first_; }
 
+    /// \brief Check if generated DHCPv6 messages shuold appear as relayed.
+    ///
+    /// \return true if generated traffic should appear as relayed.
+    bool isUseRelayedV6() const { return (v6_relay_encapsulation_level_ > 0); }
+
     /// \brief Returns template file names.
     ///
     /// \return template file names.
     std::vector<std::string> getTemplateFiles() const { return template_file_; }
 
+    /// \brief Returns location of the file containing list of MAC addresses.
+    ///
+    /// MAC addresses read from the file are used by the perfdhcp in message
+    /// exchanges with the DHCP server.
+    ///
+    /// \return Location of the file containing list of MAC addresses.
+    std::string getMacListFile() const { return mac_list_file_; }
+
+    /// \brief Returns reference to a vector of MAC addresses read from a file.
+    ///
+    /// Every MAC address is represented as a vector.
+    ///
+    /// \return Reference to a vector of vectors.
+    const MacAddrsVector& getMacsFromFile() const { return mac_list_; }
+
     /// brief Returns template offsets for xid.
     ///
     /// \return template offsets for xid.
@@ -427,7 +450,7 @@ private:
     ///
     /// \param base Base string given as -b mac=00:01:02:03:04:05.
     /// \throws isc::InvalidParameter if mac address is invalid.
-    void decodeMac(const std::string& base);
+    void decodeMacBase(const std::string& base);
 
     /// \brief Decodes base DUID provided with -b<base>.
     ///
@@ -454,6 +477,14 @@ private:
     /// \throw isc::InvalidParameter if string does not represent hex byte.
     uint8_t convertHexString(const std::string& hex_text) const;
 
+    /// \brief Opens the text file containing list of macs (one per line)
+    /// and adds them to the mac_list_ vector.
+    void loadMacs();
+
+    /// \brief Decodes a mac string into a vector of uint8_t and adds it to the
+    /// mac_list_ vector.
+    bool decodeMacString(const std::string& line);
+
     /// IP protocol version to be used, expected values are:
     /// 4 for IPv4 and 6 for IPv6, default value 0 means "not set"
     uint8_t ipversion_;
@@ -528,6 +559,14 @@ private:
     /// that are used for initiating exchanges. Template packets
     /// read from files are later tuned with variable data.
     std::vector<std::string> template_file_;
+    /// Location of a file containing a list of MAC addresses, one per line.
+    /// This can be used if you don't want to generate MAC address from a
+    /// base MAC address, but rather provide the file with a list of MAC
+    /// addresses to be randomly picked. Note that in DHCPv6 those MAC
+    /// addresses will be used to generate DUID-LL.
+    std::string mac_list_file_;
+    /// List of MAC addresses loaded from a file.
+    std::vector<std::vector<uint8_t> > mac_list_;
     /// Offset of transaction id in template files. First vector
     /// element points to offset for DISCOVER/SOLICIT messages,
     /// second element points to transaction id offset for
@@ -551,6 +590,8 @@ private:
     std::string wrapped_;
     /// Server name specified as last argument of command line.
     std::string server_name_;
+    /// Indicates how many DHCPv6 relay agents are simulated.
+    uint8_t v6_relay_encapsulation_level_;
 };
 
 } // namespace perfdhcp
