@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -13,13 +13,13 @@
 #include <dhcp/dhcp6.h>
 #include <dhcp/pkt4.h>
 #include <dhcp/pkt6.h>
+#include "../stats_mgr.h"
 
 #include <gtest/gtest.h>
 
+#include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/scoped_ptr.hpp>
 
-#include "../stats_mgr.h"
-
 using namespace std;
 using namespace isc;
 using namespace isc::dhcp;
@@ -32,6 +32,37 @@ typedef StatsMgr<dhcp::Pkt6> StatsMgr6;
 
 const uint32_t common_transid = 123;
 
+/// @brief Number of packets to be used for testing packets collecting.
+const size_t TEST_COLLECTED_PKT_NUM = 10;
+
+/// @brief DHCPv4 packet with modifiable internal values.
+///
+/// Currently the only additional modifiable value is a packet
+/// timestamp.
+class Pkt4Modifiable : public Pkt4 {
+public:
+
+    /// @brief Constructor.
+    ///
+    /// @param msg_type DHCPv4 message type.
+    /// @param transid Transaction id.
+    Pkt4Modifiable(const uint8_t msg_type, const uint32_t transid)
+        : Pkt4(msg_type, transid) {
+    }
+
+    /// @brief Modifies packet timestamp.
+    ///
+    /// @param delta Number of seconds to be added to the current
+    /// packet time. If this number is negative, the new timestamp
+    /// will point to earlier time than the original timestamp.
+    void modifyTimestamp(const long delta) {
+        timestamp_ += boost::posix_time::seconds(delta);
+    }
+};
+
+/// @brief Pointer to the Pkt4Modifiable.
+typedef boost::shared_ptr<Pkt4Modifiable> Pkt4ModifiablePtr;
+
 class StatsMgrTest : public ::testing::Test {
 public:
     StatsMgrTest() {
@@ -44,15 +75,15 @@ public:
     /// \param msg_type DHCPv4 message type.
     /// \param transid transaction id for the packet.
     /// \return DHCPv4 packet.
-    Pkt4* createPacket4(const uint8_t msg_type,
-                        const uint32_t transid) {
-        Pkt4* pkt = new Pkt4(msg_type, transid);
+    Pkt4Modifiable* createPacket4(const uint8_t msg_type,
+                                  const uint32_t transid) {
+        Pkt4Modifiable* pkt = new Pkt4Modifiable(msg_type, transid);
         // Packet timestamp is normally updated by interface
         // manager on packets reception or send. Unit tests
         // do not use interface manager so we need to do it
-        // ourselfs.
+        // ourselves.
         pkt->updateTimestamp();
-        return pkt;
+        return (pkt);
     }
 
     /// \brief Create DHCPv6 packet.
@@ -68,7 +99,7 @@ public:
         // Packet timestamp is normally updated by interface
         // manager on packets reception or send. Unit tests
         // do not use interface manager so we need to do it
-        // ourselfs.
+        // ourselves.
         pkt->updateTimestamp();
         return pkt;
     }
@@ -151,6 +182,83 @@ public:
         ASSERT_FALSE(rcvd_time.is_not_a_date_time());
     }
 
+    /// @brief This test verifies that timed out packets are collected.
+    ///
+    /// @param transid_index Index in the table of transaction ids which
+    /// points to the transaction id to be selected for the DHCPOFFER.
+    void testSendReceiveCollected(const size_t transid_index) {
+        boost::scoped_ptr<StatsMgr4> stats_mgr(new StatsMgr4());
+        // The second parameter indicates that transactions older than
+        // 2 seconds should be removed and respective packets collected.
+        stats_mgr->addExchangeStats(StatsMgr4::XCHG_DO, 2);
+
+        // Transaction ids of packets to be sent. All transaction ids
+        // belong to the same bucket (match the transid & 1023 hashing
+        // function).
+        uint32_t transid[TEST_COLLECTED_PKT_NUM] =
+            { 1, 1025, 2049, 3073, 4097, 5121, 6145, 7169, 8193, 9217 };
+
+        // Simulate sending a number of packets.
+        for (unsigned int i = 0; i < TEST_COLLECTED_PKT_NUM; ++i) {
+            Pkt4ModifiablePtr sent_packet(createPacket4(DHCPDISCOVER,
+                                                    transid[i]));
+            // For packets with low indexes we set the timestamps to
+            // 10s in the past. When DHCPOFFER is processed, the
+            // packets with timestamps older than 2s should be collected.
+            if (i < TEST_COLLECTED_PKT_NUM / 2) {
+                sent_packet->modifyTimestamp(-10);
+            }
+            ASSERT_NO_THROW(
+                stats_mgr->passSentPacket(StatsMgr4::XCHG_DO, sent_packet)
+            )  << "failure for transaction id " << transid[i];
+        }
+
+        // Create a server response for one of the packets sent.
+        Pkt4ModifiablePtr rcvd_packet(createPacket4(DHCPOFFER,
+                                                    transid[transid_index]));
+        ASSERT_NO_THROW(
+            stats_mgr->passRcvdPacket(StatsMgr4::XCHG_DO, rcvd_packet);
+        );
+
+        // There is exactly one case (transaction id) for which perfdhcp
+        // will find a packet using ordered lookup. In this case, no
+        // packets will be collected. Otherwise, for any unordered lookup
+        // all packets from a bucket should be collected.
+        if (stats_mgr->getUnorderedLookups(StatsMgr4::XCHG_DO) > 0) {
+            // All packets in the bucket having transaction id
+            // index below TEST_COLLECTED_PKT_NUM / 2 should be removed.
+            EXPECT_EQ(TEST_COLLECTED_PKT_NUM / 2,
+                      stats_mgr->getCollectedNum(StatsMgr4::XCHG_DO));
+        }
+
+        // Make sure that we can still use the StatsMgr. It is possible
+        // that the pointer to 'next sent' packet was invalidated
+        // during packet removal.
+        for (unsigned int i = 0; i < TEST_COLLECTED_PKT_NUM; ++i) {
+            // Increase transaction ids by 1 so as they don't duplicate
+            // with transaction ids of already sent packets.
+            Pkt4ModifiablePtr sent_packet(createPacket4(DHCPDISCOVER,
+                                                    transid[i] + 1));
+            Pkt4ModifiablePtr rcvd_packet(createPacket4(DHCPOFFER,
+                                                        transid[i] + 1));
+            ASSERT_NO_THROW(
+                stats_mgr->passSentPacket(StatsMgr4::XCHG_DO, sent_packet)
+            ) << "failure for transaction id " << transid[i];
+
+            ASSERT_NO_THROW(
+                stats_mgr->passRcvdPacket(StatsMgr4::XCHG_DO, rcvd_packet);
+            ) << "failure for transaction id " << transid[i];
+        }
+
+        // We should have processed TEST_COLLECTED_PKT_NUM but it is possible
+        // that one of them we couldn't match (orphan packet), because
+        // the matched packet had to be collected because of the transaction
+        // timeout. Therefore, we have to count both received packets and
+        // orphans.
+        EXPECT_EQ(TEST_COLLECTED_PKT_NUM + 1,
+                  stats_mgr->getRcvdPacketsNum(StatsMgr4::XCHG_DO) +
+                  stats_mgr->getOrphans(StatsMgr4::XCHG_DO));
+    }
 };
 
 TEST_F(StatsMgrTest, Constructor) {
@@ -238,7 +346,7 @@ TEST_F(StatsMgrTest, MultipleExchanges) {
     passMultiplePackets6(stats_mgr, StatsMgr6::XCHG_RR, DHCPV6_REQUEST,
                          request_packets_num);
 
-    // Check if all packets are successfuly passed to packet lists.
+    // Check if all packets are successfully passed to packet lists.
     EXPECT_EQ(solicit_packets_num,
               stats_mgr->getSentPacketsNum(StatsMgr6::XCHG_SA));
     EXPECT_EQ(request_packets_num,
@@ -265,6 +373,9 @@ TEST_F(StatsMgrTest, ExchangeToString) {
     EXPECT_EQ("DISCOVER-OFFER",
               StatsMgr4::exchangeToString(StatsMgr4::XCHG_DO));
     EXPECT_EQ("REQUEST-ACK", StatsMgr4::exchangeToString(StatsMgr4::XCHG_RA));
+    EXPECT_EQ("REQUEST-ACK (renewal)",
+              StatsMgr4::exchangeToString(StatsMgr4::XCHG_RNA));
+
 
     // Test DHCPv6 specific exchange names.
     EXPECT_EQ("SOLICIT-ADVERTISE",
@@ -336,6 +447,14 @@ TEST_F(StatsMgrTest, SendReceiveUnordered) {
     EXPECT_EQ(9, stats_mgr->getUnorderedLookups(StatsMgr4::XCHG_DO));
 }
 
+TEST_F(StatsMgrTest, SendReceiveCollected) {
+    // Check that the packet collection mechanism works fine
+    // for any packet returned by the server.
+    for (unsigned int i = 0; i < TEST_COLLECTED_PKT_NUM; ++i) {
+        testSendReceiveCollected(i);
+    }
+}
+
 TEST_F(StatsMgrTest, Orphans) {
     const int packets_num = 6;
     boost::scoped_ptr<StatsMgr4> stats_mgr(new StatsMgr4());
@@ -375,7 +494,7 @@ TEST_F(StatsMgrTest, Delays) {
               std::numeric_limits<double>::max());
     EXPECT_GT(stats_mgr->getMinDelay(StatsMgr4::XCHG_DO), 1);
 
-    // Max delay is supposed to the same value as mininimum
+    // Max delay is supposed to the same value as minimum
     // or maximum delay.
     EXPECT_GT(stats_mgr->getMaxDelay(StatsMgr4::XCHG_DO), 1);
 
