@@ -1,14 +1,11 @@
-// Copyright (C) 2011-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 #include <config.h>
-
-// This must be included before udp_endpoint.h
-#include <boost/asio.hpp>
-
+#include <asiolink/asio_wrapper.h>
 #include <asiolink/io_error.h>
 #include <asiolink/udp_endpoint.h>
 #include <dhcp/dhcp4.h>
@@ -883,14 +880,13 @@ IfaceMgr::send(const Pkt4Ptr& pkt) {
 }
 
 
-boost::shared_ptr<Pkt4>
-IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
+Pkt4Ptr IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
     // Sanity check for microsecond timeout.
     if (timeout_usec >= 1000000) {
         isc_throw(BadValue, "fractional timeout must be shorter than"
                   " one million microseconds");
     }
-    const SocketInfo* candidate = 0;
+    boost::scoped_ptr<SocketInfo> candidate;
     IfacePtr iface;
     fd_set sockets;
     int maxfd = 0;
@@ -975,7 +971,7 @@ IfaceMgr::receive4(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */) {
     BOOST_FOREACH(iface, ifaces_) {
         BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
             if (FD_ISSET(s.sockfd_, &sockets)) {
-                candidate = &(s);
+                candidate.reset(new SocketInfo(s));
                 break;
             }
         }
@@ -1000,7 +996,7 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
                   " one million microseconds");
     }
 
-    const SocketInfo* candidate = 0;
+    boost::scoped_ptr<SocketInfo> candidate;
     fd_set sockets;
     int maxfd = 0;
 
@@ -1085,7 +1081,7 @@ Pkt6Ptr IfaceMgr::receive6(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     BOOST_FOREACH(IfacePtr iface, ifaces_) {
         BOOST_FOREACH(SocketInfo s, iface->getSockets()) {
             if (FD_ISSET(s.sockfd_, &sockets)) {
-                candidate = &(s);
+                candidate.reset(new SocketInfo(s));
                 break;
             }
         }
