@@ -1,4 +1,4 @@
-// Copyright (C) 2012-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2012-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -22,7 +22,10 @@
 #include <boost/shared_ptr.hpp>
 #include <boost/noncopyable.hpp>
 
+#include <list>
 #include <map>
+#include <set>
+#include <utility>
 
 namespace isc {
 namespace dhcp {
@@ -246,10 +249,19 @@ public:
     /// This is an entry that represents what the client had requested,
     /// either an address or a prefix. Prefix length is 128 for regular
     /// addresses.
-    typedef std::pair<isc::asiolink::IOAddress, uint8_t> HintType;
+    typedef std::pair<isc::asiolink::IOAddress, uint8_t> ResourceType;
 
     /// @brief Container for client's hints.
-    typedef std::vector<HintType> HintContainer;
+    typedef std::vector<ResourceType> HintContainer;
+
+    /// @brief Container holding allocated prefixes or addresses.
+    typedef std::set<ResourceType> ResourceContainer;
+
+    /// @brief A tuple holding host identifier type and value.
+    typedef std::pair<Host::IdentifierType, std::vector<uint8_t> > IdentifierPair;
+
+    /// @brief Map holding values to be used as host identifiers.
+    typedef std::list<IdentifierPair> IdentifierList;
 
     /// @brief Context information for the DHCPv6 leases allocation.
     ///
@@ -273,27 +285,40 @@ public:
     /// that the big advantage of using the context structure to pass
     /// information to the allocation engine methods is that adding
     /// new information doesn't modify the API of the allocation engine.
-    struct ClientContext6 {
+    struct ClientContext6 : public boost::noncopyable {
+
+        /// @name Parameters pertaining to DHCPv6 message
+        //@{
+
+        /// @brief A pointer to the client's message
+        ///
+        /// This is used exclusively for hook purposes.
+        Pkt6Ptr query_;
+
+        /// @brief Indicates if this is a real or fake allocation.
+        ///
+        /// The real allocation is when the allocation engine is supposed
+        /// to make an update in a lease database: create new lease, or
+        /// update existing lease.
+        bool fake_allocation_;
+
         /// @brief Subnet selected for the client by the server.
         Subnet6Ptr subnet_;
 
         /// @brief Client identifier
         DuidPtr duid_;
 
-        /// @brief iaid IAID field from IA_NA or IA_PD that is being processed
-        uint32_t iaid_;
-
-        /// @brief Lease type (IA or PD)
-        Lease::Type type_;
-
         /// @brief Hardware/MAC address (if available, may be NULL)
         HWAddrPtr hwaddr_;
 
-        /// @brief client's hints
+        /// @brief A list holding host identifiers extracted from a message
+        /// received by the server.
+        IdentifierList host_identifiers_;
+
+        /// @brief A pointer to the object identifying host reservations.
         ///
-        /// There will typically be just one address, but the protocol allows
-        /// more than one address or prefix for each IA container.
-        HintContainer hints_;
+        /// May be NULL if there are no reservations.
+        ConstHostPtr host_;
 
         /// @brief A boolean value which indicates that server takes
         ///        responsibility for the forward DNS Update for this lease
@@ -314,41 +339,107 @@ public:
         /// @brief Callout handle associated with the client's message.
         hooks::CalloutHandlePtr callout_handle_;
 
-        /// @brief Indicates if this is a real or fake allocation.
-        ///
-        /// The real allocation is when the allocation engine is supposed
-        /// to make an update in a lease database: create new lease, or
-        /// update existing lease.
-        bool fake_allocation_;
+        /// @brief Holds addresses and prefixes allocated for all IAs.
+        ResourceContainer allocated_resources_;
 
-        /// @brief A pointer to any old leases that the client had before update
-        ///        but are no longer valid after the update/allocation.
-        ///
-        /// This collection is typically empty, except cases when we are doing
-        /// address reassignment, e.g. because there is a host reservation that
-        /// gives this address to someone else, so we had to return the address,
-        /// and give a new one to this client.
-        Lease6Collection old_leases_;
+        //@}
 
-        /// @brief A pointer to any leases that have changed FQDN information.
-        ///
-        /// This list may contain old versions of the leases that are still
-        /// valid. In particular, it will contain a lease if the client's
-        /// FQDN has changed.
-        Lease6Collection changed_leases_;
+        /// @brief Parameters pertaining to individual IAs.
+        struct IAContext {
 
-        /// @brief A pointer to the object identifying host reservations.
-        ///
-        /// May be NULL if there are no reservations.
-        ConstHostPtr host_;
+            /// @brief iaid IAID field from IA_NA or IA_PD that is being
+            /// processed
+            uint32_t iaid_;
 
-        /// @brief A pointer to the client's message
-        ///
-        /// This is used exclusively for hook purposes.
-        Pkt6Ptr query_;
+            /// @brief Lease type (IA or PD)
+            Lease::Type type_;
 
-        /// @brief A pointer to the IA_NA/IA_PD option to be sent in response
-        Option6IAPtr ia_rsp_;
+            /// @brief client's hints
+            ///
+            /// There will typically be just one address, but the protocol
+            /// allows more than one address or prefix for each IA container.
+            HintContainer hints_;
+
+            /// @brief A pointer to any old leases that the client had before
+            /// update but are no longer valid after the update/allocation.
+            ///
+            /// This collection is typically empty, except cases when we are
+            /// doing address reassignment, e.g. because there is a host
+            /// reservation that gives this address to someone else, so we had
+            /// to return the address, and give a new one to this client.
+            Lease6Collection old_leases_;
+
+            /// @brief A pointer to any leases that have changed FQDN
+            /// information.
+            ///
+            /// This list may contain old versions of the leases that are still
+            /// valid. In particular, it will contain a lease if the client's
+            /// FQDN has changed.
+            Lease6Collection changed_leases_;
+
+            /// @brief A pointer to the IA_NA/IA_PD option to be sent in
+            /// response
+            Option6IAPtr ia_rsp_;
+
+            /// @brief Default constructor.
+            ///
+            /// Initializes @ref type_ to @c Lease::TYPE_NA and @ref iaid_ to 0.
+            IAContext();
+
+            /// @brief Convenience method adding new hint.
+            ///
+            /// @param prefix Prefix or address.
+            /// @param prefix_len Prefix length. Default is 128 for addresses.
+            void addHint(const asiolink::IOAddress& prefix,
+                         const uint8_t prefix_len = 128);
+        };
+
+        /// @brief Container holding IA specific contexts.
+        std::vector<IAContext> ias_;
+
+        /// @brief Convenience method adding allocated prefix or address.
+        ///
+        /// @param prefix Prefix or address.
+        /// @param prefix_len Prefix length. Default is 128 for addresses.
+        void addAllocatedResource(const asiolink::IOAddress& prefix,
+                                  const uint8_t prefix_len = 128);
+
+        /// @brief Checks if specified address or prefix was allocated.
+        ///
+        /// @param prefix Prefix or address.
+        /// @param prefix_len Prefix length. Default is 128 for addresses.
+        bool isAllocated(const asiolink::IOAddress& prefix,
+                         const uint8_t prefix_len = 128) const;
+
+        /// @brief Conveniece function adding host identifier into
+        /// @ref host_identifiers_ list.
+        ///
+        /// @param id_type Identifier type.
+        /// @param identifier Identifier value.
+        void addHostIdentifier(const Host::IdentifierType& id_type,
+                               const std::vector<uint8_t>& identifier) {
+            host_identifiers_.push_back(IdentifierPair(id_type, identifier));
+        }
+
+        /// @brief Returns IA specific context for the currently processed IA.
+        ///
+        /// If IA specific context doesn't exist, it is created.
+        ///
+        /// @return Reference to IA specific context.
+        IAContext& currentIA() {
+            if (ias_.empty()) {
+                createIAContext();
+            }
+            return (ias_.back());
+        }
+
+        /// @brief Creates new IA context.
+        ///
+        /// This method should be invoked prior to processing a next IA included
+        /// in the client's message.
+        void createIAContext() {
+            ias_.push_back(IAContext());
+        };
 
         /// @brief Default constructor.
         ClientContext6();
@@ -361,9 +452,6 @@ public:
         ///
         /// @param subnet subnet the allocation should come from
         /// @param duid Client's DUID
-        /// @param iaid iaid field from the IA_NA container that client sent
-        /// @param hint a hint that the client provided
-        /// @param type lease type (IA, TA or PD)
         /// @param fwd_dns A boolean value which indicates that server takes
         ///        responsibility for the forward DNS Update for this lease
         ///        (if true).
@@ -371,13 +459,18 @@ public:
         ///        responsibility for the reverse DNS Update for this lease
         ///        (if true).
         /// @param hostname A fully qualified domain-name of the client.
-        /// @param fake_allocation is this real i.e. REQUEST (false) or just picking
-        ///        an address for SOLICIT that is not really allocated (true)
+        /// @param fake_allocation is this real i.e. REQUEST (false) or just
+        ///        picking an address for SOLICIT that is not really allocated
+        ///        (true)
+        /// @param query Pointer to the DHCPv6 message being processed.
+        /// @param callout_handle Callout handle associated with a client's
+        ///        message
         ClientContext6(const Subnet6Ptr& subnet, const DuidPtr& duid,
-                       const uint32_t iaid, const isc::asiolink::IOAddress& hint,
-                       const Lease::Type type, const bool fwd_dns, const bool
-                       rev_dns, const std::string& hostname, const bool
-                       fake_allocation);
+                       const bool fwd_dns, const bool rev_dns,
+                       const std::string& hostname, const bool fake_allocation,
+                       const Pkt6Ptr& query,
+                       const hooks::CalloutHandlePtr& callout_handle =
+                       hooks::CalloutHandlePtr());
     };
 
     /// @brief Allocates IPv6 leases for a given IA container
@@ -430,12 +523,13 @@ public:
     ///
     /// The following fields of ClientContext6 are used:
     ///
-    /// @ref ClientContext6::subnet_ subnet the allocation should come from<br/>
+    /// @ref ClientContext6::subnet_ subnet the allocation should
+    ///        come from<br/>
     /// @ref ClientContext6::duid_ Client's DUID<br/>
-    /// @ref ClientContext6::iaid_ iaid field from the IA_NA container
+    /// @ref ClientContext6::IAContext::iaid_ iaid field from the IA_NA container
     ///        that client sent<br/>
-    /// @ref ClientContext6::hints_ a hint that the client provided<br/>
-    /// @ref ClientContext6::type_ lease type (IA, TA or PD)<br/>
+    /// @ref ClientContext6::IAContext::hints_ a hint that the client provided<br/>
+    /// @ref ClientContext6::IAContext::type_ lease type (IA, TA or PD)<br/>
     /// @ref ClientContext6::fwd_dns_update_ A boolean value which indicates
     ///        that server takes responsibility for the forward DNS Update
     ///        for this lease (if true).<br/>
@@ -448,7 +542,8 @@ public:
     ///        allocated (true)<br/>
     /// @ref ClientContext6::callout_handle_ a callout handle (used in hooks). A
     ///        lease callouts will be executed if this parameter is passed.<br/>
-    /// @ref ClientContext6::old_leases_ [out] Collection to which this function
+    /// @ref ClientContext6::IAContext::old_leases_ [out] Collection to which this
+    ///        function
     ///        will append old leases. Leases are stored in the same order as in
     ///        the collection of new leases, being returned. For newly allocated
     ///        leases (not renewed) the NULL pointers are stored in this
@@ -476,9 +571,10 @@ public:
     ///
     /// @param ctx Message processing context. It holds various information
     /// extracted from the client's message and required to allocate a lease.
-    /// In particular, @ref ClientContext6::hints_ provides list of addresses or
-    /// prefixes the client had sent. @ref ClientContext6::old_leases_ will
-    /// contain removed leases in this case.
+    /// In particular, @ref ClientContext6::IAContext::hints_ provides list
+    /// of addresses or
+    /// prefixes the client had sent. @ref ClientContext6::IAContext::old_leases_
+    /// will contain removed leases in this case.
     ///
     /// @return Returns renewed lease.
     Lease6Collection renewLeases6(ClientContext6& ctx);
@@ -606,10 +702,29 @@ public:
     /// Attempts to find appropriate host reservation in HostMgr. If found, it
     /// will be set in ctx.host_.
     /// @param ctx Client context that contains all necessary information.
-    void findReservation(ClientContext6& ctx) const;
+    static void findReservation(ClientContext6& ctx);
 
 private:
 
+    /// @brief Type of the function used by @ref findReservationInternal to
+    /// retrieve reservations by subnet identifier and host identifier.
+    typedef boost::function<ConstHostPtr(const SubnetID&,
+                                         const Host::IdentifierType&,
+                                         const uint8_t*, const size_t)> HostGetFunc;
+
+    /// @brief Common function for searching host reservations.
+    ///
+    /// This is a common function called by variants of @ref findReservation
+    /// functions.
+    ///
+    /// @param ctx Reference to a @ref ClientContext6 or @ref ClientContext4.
+    /// @param host_get Pointer to the @ref HostMgr functions to be used
+    /// to retrieve reservation by subnet identifier and host identifier.
+    /// @tparam ContextType Either @ref ClientContext6 or @ref ClientContext4.
+    template<typename ContextType>
+    static void findReservationInternal(ContextType& ctx,
+                                        const HostGetFunc& host_get);
+
     /// @brief creates a lease and inserts it in LeaseMgr if necessary
     ///
     /// Creates a lease based on specified parameters and tries to insert it
@@ -767,7 +882,9 @@ private:
     /// This method attempts to extend the lease. It will call the lease6_renew
     /// or lease6_rebind hooks (depending on the client's message specified in
     /// ctx.query). The lease will be extended in LeaseMgr, unless the hooks
-    /// library will set the skip flag.
+    /// library will set the skip flag.  The old lease is added to the
+    /// the context's changed_leases_ list which allows the server to make
+    /// decisions regarding DNS updates.
     ///
     /// @param ctx client context that passes all necessary information. See
     ///        @ref ClientContext6 for details.
@@ -971,6 +1088,20 @@ public:
         /// transaction identification information.
         Pkt4Ptr query_;
 
+        /// @brief A list holding host identifiers extracted from a message
+        /// received by the server.
+        IdentifierList host_identifiers_;
+
+        /// @brief Conveniece function adding host identifier into
+        /// @ref host_identifiers_ list.
+        ///
+        /// @param id_type Identifier type.
+        /// @param identifier Identifier value.
+        void addHostIdentifier(const Host::IdentifierType& id_type,
+                               const std::vector<uint8_t>& identifier) {
+            host_identifiers_.push_back(IdentifierPair(id_type, identifier));
+        }
+
         /// @brief Default constructor.
         ClientContext4();
 
