@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -29,10 +29,10 @@ struct Lease {
 
     /// @brief Type of lease or pool
     typedef enum {
-        TYPE_NA = 0, /// the lease contains non-temporary IPv6 address
-        TYPE_TA = 1, /// the lease contains temporary IPv6 address
-        TYPE_PD = 2, /// the lease contains IPv6 prefix (for prefix delegation)
-        TYPE_V4 = 3  /// IPv4 lease
+        TYPE_NA = 0, ///< the lease contains non-temporary IPv6 address
+        TYPE_TA = 1, ///< the lease contains temporary IPv6 address
+        TYPE_PD = 2, ///< the lease contains IPv6 prefix (for prefix delegation)
+        TYPE_V4 = 3  ///< IPv4 lease
     } Type;
 
     /// @brief returns text representation of a lease type
@@ -52,8 +52,6 @@ struct Lease {
     /// @brief Expired and reclaimed lease.
     static const uint32_t STATE_EXPIRED_RECLAIMED;
 
-    //@}
-
     /// @brief Returns name(s) of the basic lease state(s).
     ///
     /// @param state A numeric value holding a state information.
@@ -479,12 +477,12 @@ struct Lease6 : public Lease {
     /// @param fqdn_rev If true, reverse DNS update is performed for a lease.
     /// @param hostname FQDN of the client which gets the lease.
     /// @param hwaddr hardware address (MAC), may be NULL
-    /// @param prefixlen An address prefix length.
+    /// @param prefixlen An address prefix length (optional, defaults to 128)
     Lease6(Lease::Type type, const isc::asiolink::IOAddress& addr, DuidPtr duid,
            uint32_t iaid, uint32_t preferred, uint32_t valid, uint32_t t1,
            uint32_t t2, SubnetID subnet_id, const bool fqdn_fwd,
            const bool fqdn_rev, const std::string& hostname,
-           const HWAddrPtr& hwaddr = HWAddrPtr(), uint8_t prefixlen = 0);
+           const HWAddrPtr& hwaddr = HWAddrPtr(), uint8_t prefixlen = 128);
 
     /// @brief Constructor
     ///
