@@ -1,4 +1,4 @@
-// Copyright (C) 2013-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2013-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -164,7 +164,14 @@ template <> void ValueParser<std::string>::build(ConstElementPtr value) {
     // Invoke common code for all specializations of build().
     buildCommon(value);
 
-    value_ = value->str();
+    // For strings we need to use stringValue() rather than str().
+    // str() returns fully escaped special characters, so
+    // single backslash would be misrepresented as "\\".
+    if (value->getType() == Element::string) {
+        value_ = value->stringValue();
+    } else {
+        value_ = value->str();
+    }
     boost::erase_all(value_, "\"");
 }
 
@@ -254,6 +261,8 @@ HooksLibrariesParser::build(ConstElementPtr value) {
 
     // This is the new syntax.  Iterate through it and get each map.
     BOOST_FOREACH(ConstElementPtr library_entry, value->listValue()) {
+        ConstElementPtr parameters;
+
         // Is it a map?
         if (library_entry->getType() != Element::map) {
             isc_throw(DhcpConfigError, "hooks library configuration error:"
@@ -264,6 +273,13 @@ HooksLibrariesParser::build(ConstElementPtr value) {
         // Iterate iterate through each element in the map.  We check
         // whether we have found a library element.
         bool lib_found = false;
+
+        string libname = "";
+
+        // Let's explicitly reset the parameters, so we won't cover old
+        // values from the previous loop round.
+        parameters.reset();
+
         BOOST_FOREACH(ConfigPair entry_item, library_entry->mapValue()) {
             if (entry_item.first == "library") {
                 if (entry_item.second->getType() != Element::string) {
@@ -275,7 +291,7 @@ HooksLibrariesParser::build(ConstElementPtr value) {
 
                 // Get the name of the library and add it to the list after
                 // removing quotes.
-                string libname = (entry_item.second)->stringValue();
+                libname = (entry_item.second)->stringValue();
 
                 // Remove leading/trailing quotes and any leading/trailing
                 // spaces.
@@ -287,10 +303,14 @@ HooksLibrariesParser::build(ConstElementPtr value) {
                         " blank (" <<
                         entry_item.second->getPosition() << ")");
                 }
-                libraries_.push_back(libname);
 
                 // Note we have found the library name.
                 lib_found = true;
+            } else {
+                // If there are parameters, let's remember them.
+                if (entry_item.first == "parameters") {
+                    parameters = entry_item.second;
+                }
             }
         }
         if (! lib_found) {
@@ -299,19 +319,26 @@ HooksLibrariesParser::build(ConstElementPtr value) {
                 " name of the library"  <<
                 " (" << library_entry->getPosition() << ")");
         }
+
+        libraries_.push_back(make_pair(libname, parameters));
     }
 
     // Check if the list of libraries has changed.  If not, nothing is done
     // - the command "DhcpN libreload" is required to reload the same
     // libraries (this prevents needless reloads when anything else in the
     // configuration is changed).
+
+    // We no longer rely on this. Parameters can change. And even if the
+    // parameters stay the same, they could point to files that could
+    // change.
     vector<string> current_libraries = HooksManager::getLibraryNames();
-    if (current_libraries == libraries_) {
+    if (current_libraries.empty() && libraries_.empty()) {
         return;
     }
 
     // Library list has changed, validate each of the libraries specified.
-    vector<string> error_libs = HooksManager::validateLibraries(libraries_);
+    vector<string> lib_names = isc::hooks::extractNames(libraries_);
+    vector<string> error_libs = HooksManager::validateLibraries(lib_names);
     if (!error_libs.empty()) {
 
         // Construct the list of libraries in error for the message.
@@ -334,15 +361,15 @@ HooksLibrariesParser::commit() {
     /// Commits the list of libraries to the configuration manager storage if
     /// the list of libraries has changed.
     if (changed_) {
-        // TODO Delete any stored CalloutHandles before reloading the
-        // libraries
+        /// @todo: Delete any stored CalloutHandles before reloading the
+        /// libraries
         HooksManager::loadLibraries(libraries_);
     }
 }
 
 // Method for testing
 void
-HooksLibrariesParser::getLibraries(std::vector<std::string>& libraries,
+HooksLibrariesParser::getLibraries(isc::hooks::HookLibsCollection& libraries,
                                    bool& changed) {
     libraries = libraries_;
     changed = changed_;
@@ -551,7 +578,7 @@ OptionDataParser::findOptionDefinition(const std::string& option_space,
     if (!def) {
         // Check if this is a vendor-option. If it is, get vendor-specific
         // definition.
-        uint32_t vendor_id = CfgOption::optionSpaceToVendorId(option_space);
+        uint32_t vendor_id = LibDHCP::optionSpaceToVendorId(option_space);
         if (vendor_id) {
             def = LibDHCP::getVendorOptionDef(u, vendor_id, search_key);
         }
@@ -608,8 +635,9 @@ OptionDataParser::createOption(ConstElementPtr option_data) {
         } else if (name_param.isSpecified() && !code_param.isSpecified()) {
             isc_throw(DhcpConfigError, "definition for the option '"
                       << space_param << "." << name_param
-                      << " does not exist ("
-                      << string_values_->getPosition("name", option_data));
+                      << "' does not exist ("
+                      << string_values_->getPosition("name", option_data)
+                      << ")");
         }
     }
 
@@ -1405,10 +1433,24 @@ D2ClientConfigParser::build(isc::data::ConstElementPtr client_config) {
                                               D2ClientConfig::
                                               DFT_OVERRIDE_CLIENT_UPDATE);
 
-        bool replace_client_name =
-            boolean_values_->getOptionalParam("replace-client-name",
-                                              D2ClientConfig::
-                                              DFT_REPLACE_CLIENT_NAME);
+        // Formerly, replace-client-name was boolean, so for now we'll support boolean
+        // values by mapping them to the appropriate mode
+        D2ClientConfig::ReplaceClientNameMode replace_client_name_mode;
+        std::string mode_str;
+        mode_str  = string_values_->getOptionalParam("replace-client-name",
+                                                     D2ClientConfig::
+                                                     DFT_REPLACE_CLIENT_NAME_MODE);
+        if (boost::iequals(mode_str, "FALSE")) {
+            // @todo add a debug log
+            replace_client_name_mode = D2ClientConfig::RCM_NEVER;
+        }
+        else if (boost::iequals(mode_str, "TRUE")) {
+            // @todo add a debug log
+            replace_client_name_mode = D2ClientConfig::RCM_WHEN_PRESENT;
+        } else {
+            replace_client_name_mode = D2ClientConfig::
+                                       stringToReplaceClientNameMode(mode_str);
+        }
 
         // Attempt to create the new client config.
         local_client_config_.reset(new D2ClientConfig(enable_updates,
@@ -1422,7 +1464,7 @@ D2ClientConfigParser::build(isc::data::ConstElementPtr client_config) {
                                                       always_include_fqdn,
                                                       override_no_update,
                                                       override_client_update,
-                                                      replace_client_name,
+                                                      replace_client_name_mode,
                                                       generated_prefix,
                                                       qualifying_suffix));
 
@@ -1444,14 +1486,14 @@ D2ClientConfigParser::createConfigParser(const std::string& config_id) {
         (config_id.compare("ncr-format") == 0) ||
         (config_id.compare("generated-prefix") == 0) ||
         (config_id.compare("sender-ip") == 0) ||
-        (config_id.compare("qualifying-suffix") == 0)) {
+        (config_id.compare("qualifying-suffix") == 0) ||
+        (config_id.compare("replace-client-name") == 0)) {
         parser = new StringParser(config_id, string_values_);
     } else if ((config_id.compare("enable-updates") == 0) ||
         (config_id.compare("always-include-fqdn") == 0) ||
         (config_id.compare("allow-client-update") == 0) ||
         (config_id.compare("override-no-update") == 0) ||
-        (config_id.compare("override-client-update") == 0) ||
-        (config_id.compare("replace-client-name") == 0)) {
+        (config_id.compare("override-client-update") == 0)) {
         parser = new BooleanParser(config_id, boolean_values_);
     } else {
         isc_throw(NotImplemented,
