@@ -1,4 +1,4 @@
-// Copyright (C) 2014-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -7,8 +7,10 @@
 #ifndef DHCPSRV_CONFIG_H
 #define DHCPSRV_CONFIG_H
 
+#include <dhcpsrv/cfg_db_access.h>
 #include <dhcpsrv/cfg_duid.h>
 #include <dhcpsrv/cfg_expiration.h>
+#include <dhcpsrv/cfg_host_operations.h>
 #include <dhcpsrv/cfg_hosts.h>
 #include <dhcpsrv/cfg_iface.h>
 #include <dhcpsrv/cfg_option.h>
@@ -278,6 +280,42 @@ public:
         return (cfg_duid_);
     }
 
+    /// @brief Returns pointer to the object holding configuration of the
+    /// lease and host database connection parameters.
+    CfgDbAccessPtr getCfgDbAccess() {
+        return (cfg_db_access_);
+    }
+
+    /// @brief Returns const pointer to the object holding configuration of
+    /// the lease and host database connection parameters.
+    ConstCfgDbAccessPtr getCfgDbAccess() const {
+        return (cfg_db_access_);
+    }
+
+    /// @brief Returns pointer to the object holding general configuration
+    /// for host reservations in DHCPv4.
+    CfgHostOperationsPtr getCfgHostOperations4() {
+        return (cfg_host_operations4_);
+    }
+
+    /// @brief Returns const pointer to the object holding general
+    /// configuration for host reservations in DHCPv4
+    ConstCfgHostOperationsPtr getCfgHostOperations4() const {
+        return (cfg_host_operations4_);
+    }
+
+    /// @brief Returns pointer to the object holding general configuration
+    /// for host reservations in DHCPv6.
+    CfgHostOperationsPtr getCfgHostOperations6() {
+        return (cfg_host_operations6_);
+    }
+
+    /// @brief Returns const pointer to the object holding general
+    /// configuration for host reservations in DHCPv6
+    ConstCfgHostOperationsPtr getCfgHostOperations6() const {
+        return (cfg_host_operations6_);
+    }
+
     //@}
 
     /// @brief Returns non-const reference to an array that stores
@@ -431,6 +469,24 @@ public:
         return (decline_timer_);
     }
 
+    /// @brief Sets DHCP4o6 IPC port
+    ///
+    /// DHCPv4-over-DHCPv6 uses a UDP socket for interserver communication,
+    /// this socket is bound and connected to this port and port + 1
+    ///
+    /// @param port port and port + 1 to use
+    void setDhcp4o6Port(uint32_t port) {
+        dhcp4o6_port_ = port;
+    }
+
+    /// @brief Returns DHCP4o6 IPC port
+    ///
+    /// See @ref setDhcp4o6Port or brief discussion.                         
+    /// @return value of DHCP4o6 IPC port
+    uint32_t getDhcp4o6Port() {
+        return (dhcp4o6_port_);
+    }
+
 private:
 
     /// @brief Sequence number identifying the configuration.
@@ -485,6 +541,18 @@ private:
     /// @brief Pointer to the configuration of the server identifier.
     CfgDUIDPtr cfg_duid_;
 
+    /// @brief Pointer to the configuration of the lease and host database
+    /// connection parameters.
+    CfgDbAccessPtr cfg_db_access_;
+
+    /// @brief Pointer to the general configuration for host reservations in
+    /// DHCPv4.
+    CfgHostOperationsPtr cfg_host_operations4_;
+
+    /// @brief Pointer to the general configuration for host reservations in
+    /// DHCPv6.
+    CfgHostOperationsPtr cfg_host_operations6_;
+
     /// @brief Pointer to the control-socket information
     isc::data::ConstElementPtr control_socket_;
 
@@ -496,6 +564,12 @@ private:
     /// This timer specifies decline probation period, the time after a declined
     /// lease is recovered back to available state. Expressed in seconds.
     uint32_t decline_timer_;
+
+    /// @brief DHCP4o6 IPC port
+    ///
+    /// DHCPv4-over-DHCPv6 uses a UDP socket for interserver communication,
+    /// this socket is bound and connected to this port and port + 1
+    uint32_t dhcp4o6_port_;
 };
 
 /// @name Pointers to the @c SrvConfig object.
