@@ -1,4 +1,4 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -847,7 +847,7 @@ public:
             }
         }
 
-        vector<string> libraries; // no libraries at this time
+        HookLibsCollection libraries; // no libraries at this time
         HooksManager::loadLibraries(libraries);
 
         // Install a callout: lease4_expire or lease6_expire.
@@ -877,7 +877,7 @@ public:
             }
         }
 
-        vector<string> libraries; // no libraries at this time
+        HookLibsCollection libraries; // no libraries at this time
         HooksManager::loadLibraries(libraries);
 
         // Install a callout: lease4_expire or lease6_expire.
@@ -905,7 +905,7 @@ public:
             expire(i, 2000 - i);
         }
 
-        vector<string> libraries;
+        HookLibsCollection libraries;
         HooksManager::loadLibraries(libraries);
 
         // Install a callout: lease4_expire or lease6_expire. Each callout
@@ -1415,14 +1415,13 @@ ExpirationAllocEngine6Test::testReclaimReusedLeases(const uint16_t msg_type,
 
     for (unsigned int i = 0; i < TEST_LEASES_NUM; ++i) {
         // Build the context.
-        AllocEngine::ClientContext6 ctx(subnet, leases_[i]->duid_, 1,
-                                        leases_[i]->addr_,
-                                        Lease::TYPE_NA,
+        AllocEngine::ClientContext6 ctx(subnet, leases_[i]->duid_,
                                         false, false,
                                         leases_[i]->hostname_,
-                                        msg_type == DHCPV6_SOLICIT);
-        // Query is needed for logging purposes.
-        ctx.query_.reset(new Pkt6(msg_type, 0x1234));
+                                        msg_type == DHCPV6_SOLICIT,
+                                        Pkt6Ptr(new Pkt6(msg_type, 0x1234)));
+        ctx.currentIA().iaid_ = 1;
+        ctx.currentIA().hints_.push_back(std::make_pair(leases_[i]->addr_, 128));
 
         // Depending on the message type, we will call a different function.
         if (msg_type == DHCPV6_RENEW) {
@@ -1854,7 +1853,7 @@ void
 ExpirationAllocEngine4Test::setUniqueClientId(const uint16_t index) {
     std::ostringstream clientid_s;
     clientid_s << "AA:BB:" << std::setw(2) << std::setfill('0')
-        << (index >> 16) << ":" << std::setw(2) << std::setfill('0')
+        << (index >> 8) << ":" << std::setw(2) << std::setfill('0')
         << (index & 0x00FF);
     ClientIdPtr client_id(ClientId::fromText(clientid_s.str()));
     leases_[index]->client_id_ = client_id;
