@@ -1,10 +1,11 @@
-// Copyright (C) 2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-#include <boost/asio.hpp>
+#include <config.h>
+#include <asiolink/asio_wrapper.h>
 #include <asiolink/io_service.h>
 #include <dhcp/iface_mgr.h>
 #include <dhcpsrv/dhcpsrv_log.h>
@@ -13,7 +14,9 @@
 #include <util/threads/sync.h>
 #include <util/threads/thread.h>
 #include <util/watch_socket.h>
+
 #include <boost/bind.hpp>
+
 #include <utility>
 
 using namespace isc;
