@@ -40,7 +40,7 @@
 #ifndef YY_YY_PARSER_H_INCLUDED
 # define YY_YY_PARSER_H_INCLUDED
 // //                    "%code requires" blocks.
-#line 16 "parser.yy" // lalr1.cc:392
+#line 16 "parser.yy" // lalr1.cc:377
 
 #include <string>
 #include <eval/token.h>
@@ -51,7 +51,7 @@
 using namespace isc::dhcp;
 using namespace isc::eval;
 
-#line 55 "parser.h" // lalr1.cc:392
+#line 55 "parser.h" // lalr1.cc:377
 
 # include <cassert>
 # include <cstdlib> // std::abort
@@ -126,9 +126,9 @@ using namespace isc::eval;
 # define YYDEBUG 1
 #endif
 
-#line 13 "parser.yy" // lalr1.cc:392
+#line 13 "parser.yy" // lalr1.cc:377
 namespace isc { namespace eval {
-#line 132 "parser.h" // lalr1.cc:392
+#line 132 "parser.h" // lalr1.cc:377
 
 
 
@@ -298,15 +298,34 @@ namespace isc { namespace eval {
       // option_repr_type
       char dummy1[sizeof(TokenOption::RepresentationType)];
 
+      // pkt4_field
+      char dummy2[sizeof(TokenPkt4::FieldType)];
+
+      // pkt6_field
+      char dummy3[sizeof(TokenPkt6::FieldType)];
+
+      // pkt_metadata
+      char dummy4[sizeof(TokenPkt::MetadataType)];
+
+      // relay6_field
+      char dummy5[sizeof(TokenRelay6Field::FieldType)];
+
       // "constant string"
       // "integer"
       // "constant hexstring"
       // "option name"
-      // TOKEN
-      char dummy2[sizeof(std::string)];
+      // "ip address"
+      char dummy6[sizeof(std::string)];
 
       // option_code
-      char dummy3[sizeof(uint16_t)];
+      char dummy7[sizeof(uint16_t)];
+
+      // integer_expr
+      // enterprise_id
+      char dummy8[sizeof(uint32_t)];
+
+      // nest_level
+      char dummy9[sizeof(uint8_t)];
 };
 
     /// Symbol semantic values.
@@ -330,23 +349,53 @@ namespace isc { namespace eval {
       enum yytokentype
       {
         TOKEN_END = 0,
-        TOKEN_EQUAL = 258,
-        TOKEN_OPTION = 259,
-        TOKEN_SUBSTRING = 260,
-        TOKEN_TEXT = 261,
-        TOKEN_HEX = 262,
-        TOKEN_ALL = 263,
-        TOKEN_DOT = 264,
-        TOKEN_COMA = 265,
-        TOKEN_LPAREN = 266,
-        TOKEN_RPAREN = 267,
-        TOKEN_LBRACKET = 268,
-        TOKEN_RBRACKET = 269,
-        TOKEN_STRING = 270,
-        TOKEN_INTEGER = 271,
-        TOKEN_HEXSTRING = 272,
-        TOKEN_OPTION_NAME = 273,
-        TOKEN_TOKEN = 274
+        TOKEN_LPAREN = 258,
+        TOKEN_RPAREN = 259,
+        TOKEN_NOT = 260,
+        TOKEN_AND = 261,
+        TOKEN_OR = 262,
+        TOKEN_EQUAL = 263,
+        TOKEN_OPTION = 264,
+        TOKEN_RELAY4 = 265,
+        TOKEN_RELAY6 = 266,
+        TOKEN_PEERADDR = 267,
+        TOKEN_LINKADDR = 268,
+        TOKEN_LBRACKET = 269,
+        TOKEN_RBRACKET = 270,
+        TOKEN_DOT = 271,
+        TOKEN_TEXT = 272,
+        TOKEN_HEX = 273,
+        TOKEN_EXISTS = 274,
+        TOKEN_PKT = 275,
+        TOKEN_IFACE = 276,
+        TOKEN_SRC = 277,
+        TOKEN_DST = 278,
+        TOKEN_LEN = 279,
+        TOKEN_PKT4 = 280,
+        TOKEN_CHADDR = 281,
+        TOKEN_HLEN = 282,
+        TOKEN_HTYPE = 283,
+        TOKEN_CIADDR = 284,
+        TOKEN_GIADDR = 285,
+        TOKEN_YIADDR = 286,
+        TOKEN_SIADDR = 287,
+        TOKEN_SUBSTRING = 288,
+        TOKEN_ALL = 289,
+        TOKEN_COMA = 290,
+        TOKEN_CONCAT = 291,
+        TOKEN_PKT6 = 292,
+        TOKEN_MSGTYPE = 293,
+        TOKEN_TRANSID = 294,
+        TOKEN_VENDOR_CLASS = 295,
+        TOKEN_VENDOR = 296,
+        TOKEN_ANY = 297,
+        TOKEN_DATA = 298,
+        TOKEN_ENTERPRISE = 299,
+        TOKEN_STRING = 300,
+        TOKEN_INTEGER = 301,
+        TOKEN_HEXSTRING = 302,
+        TOKEN_OPTION_NAME = 303,
+        TOKEN_IP_ADDRESS = 304
       };
     };
 
@@ -386,10 +435,22 @@ namespace isc { namespace eval {
 
   basic_symbol (typename Base::kind_type t, const TokenOption::RepresentationType v, const location_type& l);
 
+  basic_symbol (typename Base::kind_type t, const TokenPkt4::FieldType v, const location_type& l);
+
+  basic_symbol (typename Base::kind_type t, const TokenPkt6::FieldType v, const location_type& l);
+
+  basic_symbol (typename Base::kind_type t, const TokenPkt::MetadataType v, const location_type& l);
+
+  basic_symbol (typename Base::kind_type t, const TokenRelay6Field::FieldType v, const location_type& l);
+
   basic_symbol (typename Base::kind_type t, const std::string v, const location_type& l);
 
   basic_symbol (typename Base::kind_type t, const uint16_t v, const location_type& l);
 
+  basic_symbol (typename Base::kind_type t, const uint32_t v, const location_type& l);
+
+  basic_symbol (typename Base::kind_type t, const uint8_t v, const location_type& l);
+
 
       /// Constructor for symbols with semantic value.
       basic_symbol (typename Base::kind_type t,
@@ -461,6 +522,26 @@ namespace isc { namespace eval {
     symbol_type
     make_END (const location_type& l);
 
+    static inline
+    symbol_type
+    make_LPAREN (const location_type& l);
+
+    static inline
+    symbol_type
+    make_RPAREN (const location_type& l);
+
+    static inline
+    symbol_type
+    make_NOT (const location_type& l);
+
+    static inline
+    symbol_type
+    make_AND (const location_type& l);
+
+    static inline
+    symbol_type
+    make_OR (const location_type& l);
+
     static inline
     symbol_type
     make_EQUAL (const location_type& l);
@@ -471,7 +552,31 @@ namespace isc { namespace eval {
 
     static inline
     symbol_type
-    make_SUBSTRING (const location_type& l);
+    make_RELAY4 (const location_type& l);
+
+    static inline
+    symbol_type
+    make_RELAY6 (const location_type& l);
+
+    static inline
+    symbol_type
+    make_PEERADDR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_LINKADDR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_LBRACKET (const location_type& l);
+
+    static inline
+    symbol_type
+    make_RBRACKET (const location_type& l);
+
+    static inline
+    symbol_type
+    make_DOT (const location_type& l);
 
     static inline
     symbol_type
@@ -483,11 +588,67 @@ namespace isc { namespace eval {
 
     static inline
     symbol_type
-    make_ALL (const location_type& l);
+    make_EXISTS (const location_type& l);
 
     static inline
     symbol_type
-    make_DOT (const location_type& l);
+    make_PKT (const location_type& l);
+
+    static inline
+    symbol_type
+    make_IFACE (const location_type& l);
+
+    static inline
+    symbol_type
+    make_SRC (const location_type& l);
+
+    static inline
+    symbol_type
+    make_DST (const location_type& l);
+
+    static inline
+    symbol_type
+    make_LEN (const location_type& l);
+
+    static inline
+    symbol_type
+    make_PKT4 (const location_type& l);
+
+    static inline
+    symbol_type
+    make_CHADDR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_HLEN (const location_type& l);
+
+    static inline
+    symbol_type
+    make_HTYPE (const location_type& l);
+
+    static inline
+    symbol_type
+    make_CIADDR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_GIADDR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_YIADDR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_SIADDR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_SUBSTRING (const location_type& l);
+
+    static inline
+    symbol_type
+    make_ALL (const location_type& l);
 
     static inline
     symbol_type
@@ -495,19 +656,39 @@ namespace isc { namespace eval {
 
     static inline
     symbol_type
-    make_LPAREN (const location_type& l);
+    make_CONCAT (const location_type& l);
 
     static inline
     symbol_type
-    make_RPAREN (const location_type& l);
+    make_PKT6 (const location_type& l);
 
     static inline
     symbol_type
-    make_LBRACKET (const location_type& l);
+    make_MSGTYPE (const location_type& l);
 
     static inline
     symbol_type
-    make_RBRACKET (const location_type& l);
+    make_TRANSID (const location_type& l);
+
+    static inline
+    symbol_type
+    make_VENDOR_CLASS (const location_type& l);
+
+    static inline
+    symbol_type
+    make_VENDOR (const location_type& l);
+
+    static inline
+    symbol_type
+    make_ANY (const location_type& l);
+
+    static inline
+    symbol_type
+    make_DATA (const location_type& l);
+
+    static inline
+    symbol_type
+    make_ENTERPRISE (const location_type& l);
 
     static inline
     symbol_type
@@ -527,7 +708,7 @@ namespace isc { namespace eval {
 
     static inline
     symbol_type
-    make_TOKEN (const std::string& v, const location_type& l);
+    make_IP_ADDRESS (const std::string& v, const location_type& l);
 
 
     /// Build a parser object.
@@ -596,7 +777,7 @@ namespace isc { namespace eval {
     // Tables.
   // YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
   // STATE-NUM.
-  static const signed char yypact_[];
+  static const short int yypact_[];
 
   // YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
   // Performed when YYTABLE does not specify something else to do.  Zero
@@ -607,14 +788,14 @@ namespace isc { namespace eval {
   static const signed char yypgoto_[];
 
   // YYDEFGOTO[NTERM-NUM].
-  static const signed char yydefgoto_[];
+  static const short int yydefgoto_[];
 
   // YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
   // positive, shift that token.  If negative, reduce the rule whose
   // number is the opposite.  If YYTABLE_NINF, syntax error.
   static const unsigned char yytable_[];
 
-  static const signed char yycheck_[];
+  static const short int yycheck_[];
 
   // YYSTOS[STATE-NUM] -- The (internal number of the) accessing
   // symbol of state STATE-NUM.
@@ -635,7 +816,7 @@ namespace isc { namespace eval {
     static const char* const yytname_[];
 #if YYDEBUG
   // YYRLINE[YYN] -- Source line where rule number YYN was defined.
-  static const unsigned char yyrline_[];
+  static const unsigned short int yyrline_[];
     /// Report on the debug stream that the rule \a r is going to be reduced.
     virtual void yy_reduce_print_ (int r);
     /// Print the state stack on the debug stream.
@@ -734,12 +915,12 @@ namespace isc { namespace eval {
     enum
     {
       yyeof_ = 0,
-      yylast_ = 24,     ///< Last index in yytable_.
-      yynnts_ = 8,  ///< Number of nonterminal symbols.
-      yyfinal_ = 11, ///< Termination state number.
+      yylast_ = 174,     ///< Last index in yytable_.
+      yynnts_ = 15,  ///< Number of nonterminal symbols.
+      yyfinal_ = 35, ///< Termination state number.
       yyterror_ = 1,
       yyerrcode_ = 256,
-      yyntokens_ = 20  ///< Number of tokens.
+      yyntokens_ = 50  ///< Number of tokens.
     };
 
 
@@ -783,9 +964,12 @@ namespace isc { namespace eval {
        2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
        2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
        5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
-      15,    16,    17,    18,    19
+      15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
+      25,    26,    27,    28,    29,    30,    31,    32,    33,    34,
+      35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
+      45,    46,    47,    48,    49
     };
-    const unsigned int user_token_number_max_ = 274;
+    const unsigned int user_token_number_max_ = 304;
     const token_number_type undef_token_ = 2;
 
     if (static_cast<int>(t) <= yyeof_)
@@ -818,22 +1002,47 @@ namespace isc { namespace eval {
   {
       switch (other.type_get ())
     {
-      case 25: // option_repr_type
+      case 56: // option_repr_type
         value.copy< TokenOption::RepresentationType > (other.value);
         break;
 
-      case 15: // "constant string"
-      case 16: // "integer"
-      case 17: // "constant hexstring"
-      case 18: // "option name"
-      case 19: // TOKEN
+      case 60: // pkt4_field
+        value.copy< TokenPkt4::FieldType > (other.value);
+        break;
+
+      case 61: // pkt6_field
+        value.copy< TokenPkt6::FieldType > (other.value);
+        break;
+
+      case 58: // pkt_metadata
+        value.copy< TokenPkt::MetadataType > (other.value);
+        break;
+
+      case 62: // relay6_field
+        value.copy< TokenRelay6Field::FieldType > (other.value);
+        break;
+
+      case 45: // "constant string"
+      case 46: // "integer"
+      case 47: // "constant hexstring"
+      case 48: // "option name"
+      case 49: // "ip address"
         value.copy< std::string > (other.value);
         break;
 
-      case 24: // option_code
+      case 55: // option_code
         value.copy< uint16_t > (other.value);
         break;
 
+      case 54: // integer_expr
+      case 59: // enterprise_id
+        value.copy< uint32_t > (other.value);
+        break;
+
+      case 57: // nest_level
+        value.copy< uint8_t > (other.value);
+        break;
+
       default:
         break;
     }
@@ -851,22 +1060,47 @@ namespace isc { namespace eval {
     (void) v;
       switch (this->type_get ())
     {
-      case 25: // option_repr_type
+      case 56: // option_repr_type
         value.copy< TokenOption::RepresentationType > (v);
         break;
 
-      case 15: // "constant string"
-      case 16: // "integer"
-      case 17: // "constant hexstring"
-      case 18: // "option name"
-      case 19: // TOKEN
+      case 60: // pkt4_field
+        value.copy< TokenPkt4::FieldType > (v);
+        break;
+
+      case 61: // pkt6_field
+        value.copy< TokenPkt6::FieldType > (v);
+        break;
+
+      case 58: // pkt_metadata
+        value.copy< TokenPkt::MetadataType > (v);
+        break;
+
+      case 62: // relay6_field
+        value.copy< TokenRelay6Field::FieldType > (v);
+        break;
+
+      case 45: // "constant string"
+      case 46: // "integer"
+      case 47: // "constant hexstring"
+      case 48: // "option name"
+      case 49: // "ip address"
         value.copy< std::string > (v);
         break;
 
-      case 24: // option_code
+      case 55: // option_code
         value.copy< uint16_t > (v);
         break;
 
+      case 54: // integer_expr
+      case 59: // enterprise_id
+        value.copy< uint32_t > (v);
+        break;
+
+      case 57: // nest_level
+        value.copy< uint8_t > (v);
+        break;
+
       default:
         break;
     }
@@ -889,6 +1123,34 @@ namespace isc { namespace eval {
     , location (l)
   {}
 
+  template <typename Base>
+  EvalParser::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, const TokenPkt4::FieldType v, const location_type& l)
+    : Base (t)
+    , value (v)
+    , location (l)
+  {}
+
+  template <typename Base>
+  EvalParser::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, const TokenPkt6::FieldType v, const location_type& l)
+    : Base (t)
+    , value (v)
+    , location (l)
+  {}
+
+  template <typename Base>
+  EvalParser::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, const TokenPkt::MetadataType v, const location_type& l)
+    : Base (t)
+    , value (v)
+    , location (l)
+  {}
+
+  template <typename Base>
+  EvalParser::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, const TokenRelay6Field::FieldType v, const location_type& l)
+    : Base (t)
+    , value (v)
+    , location (l)
+  {}
+
   template <typename Base>
   EvalParser::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, const std::string v, const location_type& l)
     : Base (t)
@@ -903,6 +1165,20 @@ namespace isc { namespace eval {
     , location (l)
   {}
 
+  template <typename Base>
+  EvalParser::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, const uint32_t v, const location_type& l)
+    : Base (t)
+    , value (v)
+    , location (l)
+  {}
+
+  template <typename Base>
+  EvalParser::basic_symbol<Base>::basic_symbol (typename Base::kind_type t, const uint8_t v, const location_type& l)
+    : Base (t)
+    , value (v)
+    , location (l)
+  {}
+
 
   template <typename Base>
   inline
@@ -929,22 +1205,47 @@ namespace isc { namespace eval {
     // Type destructor.
     switch (yytype)
     {
-      case 25: // option_repr_type
+      case 56: // option_repr_type
         value.template destroy< TokenOption::RepresentationType > ();
         break;
 
-      case 15: // "constant string"
-      case 16: // "integer"
-      case 17: // "constant hexstring"
-      case 18: // "option name"
-      case 19: // TOKEN
+      case 60: // pkt4_field
+        value.template destroy< TokenPkt4::FieldType > ();
+        break;
+
+      case 61: // pkt6_field
+        value.template destroy< TokenPkt6::FieldType > ();
+        break;
+
+      case 58: // pkt_metadata
+        value.template destroy< TokenPkt::MetadataType > ();
+        break;
+
+      case 62: // relay6_field
+        value.template destroy< TokenRelay6Field::FieldType > ();
+        break;
+
+      case 45: // "constant string"
+      case 46: // "integer"
+      case 47: // "constant hexstring"
+      case 48: // "option name"
+      case 49: // "ip address"
         value.template destroy< std::string > ();
         break;
 
-      case 24: // option_code
+      case 55: // option_code
         value.template destroy< uint16_t > ();
         break;
 
+      case 54: // integer_expr
+      case 59: // enterprise_id
+        value.template destroy< uint32_t > ();
+        break;
+
+      case 57: // nest_level
+        value.template destroy< uint8_t > ();
+        break;
+
       default:
         break;
     }
@@ -968,22 +1269,47 @@ namespace isc { namespace eval {
     super_type::move(s);
       switch (this->type_get ())
     {
-      case 25: // option_repr_type
+      case 56: // option_repr_type
         value.move< TokenOption::RepresentationType > (s.value);
         break;
 
-      case 15: // "constant string"
-      case 16: // "integer"
-      case 17: // "constant hexstring"
-      case 18: // "option name"
-      case 19: // TOKEN
+      case 60: // pkt4_field
+        value.move< TokenPkt4::FieldType > (s.value);
+        break;
+
+      case 61: // pkt6_field
+        value.move< TokenPkt6::FieldType > (s.value);
+        break;
+
+      case 58: // pkt_metadata
+        value.move< TokenPkt::MetadataType > (s.value);
+        break;
+
+      case 62: // relay6_field
+        value.move< TokenRelay6Field::FieldType > (s.value);
+        break;
+
+      case 45: // "constant string"
+      case 46: // "integer"
+      case 47: // "constant hexstring"
+      case 48: // "option name"
+      case 49: // "ip address"
         value.move< std::string > (s.value);
         break;
 
-      case 24: // option_code
+      case 55: // option_code
         value.move< uint16_t > (s.value);
         break;
 
+      case 54: // integer_expr
+      case 59: // enterprise_id
+        value.move< uint32_t > (s.value);
+        break;
+
+      case 57: // nest_level
+        value.move< uint8_t > (s.value);
+        break;
+
       default:
         break;
     }
@@ -1040,7 +1366,10 @@ namespace isc { namespace eval {
     yytoken_number_[] =
     {
        0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
-     265,   266,   267,   268,   269,   270,   271,   272,   273,   274
+     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
+     275,   276,   277,   278,   279,   280,   281,   282,   283,   284,
+     285,   286,   287,   288,   289,   290,   291,   292,   293,   294,
+     295,   296,   297,   298,   299,   300,   301,   302,   303,   304
     };
     return static_cast<token_type> (yytoken_number_[type]);
   }
@@ -1051,54 +1380,6 @@ namespace isc { namespace eval {
     return symbol_type (token::TOKEN_END, l);
   }
 
-  EvalParser::symbol_type
-  EvalParser::make_EQUAL (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_EQUAL, l);
-  }
-
-  EvalParser::symbol_type
-  EvalParser::make_OPTION (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_OPTION, l);
-  }
-
-  EvalParser::symbol_type
-  EvalParser::make_SUBSTRING (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_SUBSTRING, l);
-  }
-
-  EvalParser::symbol_type
-  EvalParser::make_TEXT (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_TEXT, l);
-  }
-
-  EvalParser::symbol_type
-  EvalParser::make_HEX (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_HEX, l);
-  }
-
-  EvalParser::symbol_type
-  EvalParser::make_ALL (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_ALL, l);
-  }
-
-  EvalParser::symbol_type
-  EvalParser::make_DOT (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_DOT, l);
-  }
-
-  EvalParser::symbol_type
-  EvalParser::make_COMA (const location_type& l)
-  {
-    return symbol_type (token::TOKEN_COMA, l);
-  }
-
   EvalParser::symbol_type
   EvalParser::make_LPAREN (const location_type& l)
   {
@@ -1111,6 +1392,60 @@ namespace isc { namespace eval {
     return symbol_type (token::TOKEN_RPAREN, l);
   }
 
+  EvalParser::symbol_type
+  EvalParser::make_NOT (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_NOT, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_AND (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_AND, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_OR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_OR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_EQUAL (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_EQUAL, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_OPTION (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_OPTION, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_RELAY4 (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_RELAY4, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_RELAY6 (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_RELAY6, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_PEERADDR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_PEERADDR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_LINKADDR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_LINKADDR, l);
+  }
+
   EvalParser::symbol_type
   EvalParser::make_LBRACKET (const location_type& l)
   {
@@ -1123,6 +1458,180 @@ namespace isc { namespace eval {
     return symbol_type (token::TOKEN_RBRACKET, l);
   }
 
+  EvalParser::symbol_type
+  EvalParser::make_DOT (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_DOT, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_TEXT (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_TEXT, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_HEX (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_HEX, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_EXISTS (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_EXISTS, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_PKT (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_PKT, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_IFACE (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_IFACE, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_SRC (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_SRC, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_DST (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_DST, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_LEN (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_LEN, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_PKT4 (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_PKT4, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_CHADDR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_CHADDR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_HLEN (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_HLEN, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_HTYPE (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_HTYPE, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_CIADDR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_CIADDR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_GIADDR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_GIADDR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_YIADDR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_YIADDR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_SIADDR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_SIADDR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_SUBSTRING (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_SUBSTRING, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_ALL (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_ALL, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_COMA (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_COMA, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_CONCAT (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_CONCAT, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_PKT6 (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_PKT6, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_MSGTYPE (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_MSGTYPE, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_TRANSID (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_TRANSID, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_VENDOR_CLASS (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_VENDOR_CLASS, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_VENDOR (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_VENDOR, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_ANY (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_ANY, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_DATA (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_DATA, l);
+  }
+
+  EvalParser::symbol_type
+  EvalParser::make_ENTERPRISE (const location_type& l)
+  {
+    return symbol_type (token::TOKEN_ENTERPRISE, l);
+  }
+
   EvalParser::symbol_type
   EvalParser::make_STRING (const std::string& v, const location_type& l)
   {
@@ -1148,15 +1657,15 @@ namespace isc { namespace eval {
   }
 
   EvalParser::symbol_type
-  EvalParser::make_TOKEN (const std::string& v, const location_type& l)
+  EvalParser::make_IP_ADDRESS (const std::string& v, const location_type& l)
   {
-    return symbol_type (token::TOKEN_TOKEN, v, l);
+    return symbol_type (token::TOKEN_IP_ADDRESS, v, l);
   }
 
 
-#line 13 "parser.yy" // lalr1.cc:392
+#line 13 "parser.yy" // lalr1.cc:377
 } } // isc::eval
-#line 1160 "parser.h" // lalr1.cc:392
+#line 1669 "parser.h" // lalr1.cc:377
 
 
 
