@@ -1,4 +1,4 @@
-// Copyright (C) 2011-2015 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2011-2016 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -474,7 +474,7 @@ ForwardTest::checkPushAndPop(int family, int type, int protocol,
                                        false));
         setNonBlock(client_sock.fd, true);
         // This connect would "fail" due to EINPROGRESS.  Ignore it for now.
-        connect(client_sock.fd, local.first, local.second);
+        static_cast<void>(connect(client_sock.fd, local.first, local.second));
         sockaddr_storage ss;
         socklen_t salen = sizeof(ss);
         server_sock.reset(accept(sock.fd, convertSockAddr(&ss), &salen));
@@ -532,18 +532,18 @@ ForwardTest::checkPushAndPop(int family, int type, int protocol,
     // reliable.  Instead we impose some reasonably large upper time limit of
     // blocking (normally it shouldn't even block at all; the limit is to
     // force the test to stop even if there's some bug and recv fails).
-    char recvbuf[sizeof(TEST_DATA)];
+    char recvbuf[sizeof(TEST_DATA) + 1] = { };
     sockaddr_storage ss;
     socklen_t sa_len = sizeof(ss);
     if (protocol == IPPROTO_UDP) {
-        EXPECT_EQ(sizeof(recvbuf),
-                  recvfrom(fwd_fd, recvbuf, sizeof(recvbuf),
+        EXPECT_EQ(sizeof(TEST_DATA),
+                  recvfrom(fwd_fd, recvbuf, sizeof(TEST_DATA),
                            setRecvDelay(fwd_fd), convertSockAddr(&ss),
                            &sa_len));
     } else {
         setNonBlock(client_sock.fd, false);
-        EXPECT_EQ(sizeof(recvbuf),
-                  recv(client_sock.fd, recvbuf, sizeof(recvbuf),
+        EXPECT_EQ(sizeof(TEST_DATA),
+                  recv(client_sock.fd, recvbuf, sizeof(TEST_DATA),
                        setRecvDelay(client_sock.fd)));
     }
     EXPECT_EQ(string(TEST_DATA), string(recvbuf));
