@@ -1,5 +1,5 @@
 #!/bin/sh
-# Copyright (C) 2014-2015 Internet Systems Consortium, Inc. ("ISC")
+# Copyright (C) 2014-2016 Internet Systems Consortium, Inc. ("ISC")
 #
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -27,4 +27,4 @@ echo "Replacing \@localstatedir\@ with ${localstatedir}"
 echo "Input file: $1"
 echo "Output file: $2"
 
-sed -e "s+\@localstatedir\@+${localstatedir}+g; s+\/usr/local+${prefix}+g; s+\${prefix}/etc+${sysconfdir}+g" $1 > $2
+sed -e "s+\@localstatedir\@+${localstatedir}+g; s+\@prefix\@+${prefix}+g; s+\@sysconfdir\@+${sysconfdir}+g" $1 > $2
